@@ -1,6 +1,6 @@
 ;;; install.el --- Emacs Lisp package install utility
 
-;; Copyright (C) 1996,1997,1998,1999 Free Software Foundation, Inc.
+;; Copyright (C) 1996,1997,1998,1999,2001 Free Software Foundation, Inc.
 
 ;; Author: MORIOKA Tomohiko <morioka@jaist.ac.jp>
 ;; Created: 1996/08/18
@@ -62,6 +62,7 @@
 	    (if (and (file-exists-p full-path) overwrite)
 		(delete-file full-path))
 	    (copy-file src-file full-path t t)
+	    (set-file-modes full-path install-overwritten-file-modes)
 	    (if move
 		(catch 'tag
 		  (while (and (file-exists-p src-file)
@@ -74,7 +75,8 @@
 	    (princ (format "%s -> %s\n" file dest)))))))
 
 (defun install-files (files src dest &optional move overwrite just-print)
-  (or (file-exists-p dest)
+  (or just-print
+      (file-exists-p dest)
       (make-directory dest t))
   (mapcar
    (function
@@ -86,7 +88,7 @@
 ;;; @@ install Emacs Lisp files
 ;;;
 
-(defun install-elisp-module (module src dest &optional just-print)
+(defun install-elisp-module (module src dest &optional just-print del-elc)
   (let (el-file elc-file)
     (let ((name (symbol-name module)))
       (setq el-file (concat name ".el"))
@@ -100,16 +102,23 @@
 	    (if (file-exists-p full-path)
 		(delete-file full-path))
 	    (copy-file src-file full-path t t)
+	    (set-file-modes full-path install-overwritten-file-modes)
 	    (princ (format "%s -> %s\n" el-file dest)))))
       (setq src-file (expand-file-name elc-file src))
       (if (not (file-exists-p src-file))
-	  nil 
+	  (let ((full-path (expand-file-name elc-file dest)))
+	    (if (and del-elc (file-exists-p full-path))
+		(if just-print
+		    (princ (format "%s -> to be deleted\n" full-path))
+		  (delete-file full-path)
+		  (princ (format "%s -> deleted\n" full-path)))))
 	(if just-print
 	    (princ (format "%s -> %s\n" elc-file dest))
 	  (let ((full-path (expand-file-name elc-file dest)))
             (if (file-exists-p full-path)
                 (delete-file full-path))
 	    (copy-file src-file full-path t t)
+	    (set-file-modes full-path install-overwritten-file-modes)
 	    (catch 'tag
 	      (while (file-exists-p src-file)
 		(condition-case err
@@ -119,13 +128,14 @@
 		  (error (princ (format "%s\n" (nth 1 err)))))))
 	    (princ (format "%s -> %s\n" elc-file dest))))))))
 
-(defun install-elisp-modules (modules src dest &optional just-print)
-  (or (file-exists-p dest)
+(defun install-elisp-modules (modules src dest &optional just-print del-elc)
+  (or just-print
+      (file-exists-p dest)
       (make-directory dest t))
   (mapcar
    (function
     (lambda (module)
-      (install-elisp-module module src dest just-print)))
+      (install-elisp-module module src dest just-print del-elc)))
    modules))
 
 
