@@ -267,7 +267,7 @@ HIST, if non-nil, specifies a history list
 DEF, if non-nil, is the default value.
 
 Completion ignores case if the ambient value of
-  `completion-ignore-case' is non-nil."  
+  `completion-ignore-case' is non-nil."
 	(let ((string (si:completing-read prompt table predicate
 					  require-match init hist)))
 	  (if (and (string= string "") def)
@@ -704,49 +704,97 @@ Elements of LIST that are not conses are ignored."
 	     (throw 'found (car list))))
       (setq list (cdr list)))))
 
-;; XEmacs 19.13 and later: (remassq KEY LIST)
-(defun-maybe remassq (key list)
-  "Delete by side effect any elements of LIST whose car is `eq' to KEY.
-The modified LIST is returned.  If the first member of LIST has a car
-that is `eq' to KEY, there is no way to remove it by side effect;
-therefore, write `(setq foo (remassq key foo))' to be sure of changing
-the value of `foo'."
-  (if (setq key (assq key list))
-      (delq key list)
-    list))
-
-;; XEmacs 19.13 and later: (remassoc KEY LIST)
-(defun-maybe remassoc (key list)
-  "Delete by side effect any elements of LIST whose car is `equal' to KEY.
-The modified LIST is returned.  If the first member of LIST has a car
+;; XEmacs 19.13 and later: (remassoc KEY ALIST)
+(defun-maybe remassoc (key alist)
+  "Delete by side effect any elements of ALIST whose car is `equal' to KEY.
+The modified ALIST is returned.  If the first member of ALIST has a car
 that is `equal' to KEY, there is no way to remove it by side effect;
 therefore, write `(setq foo (remassoc key foo))' to be sure of changing
 the value of `foo'."
-  (if (setq key (assoc key list))
-      (delq key list)
-    list))
+  (while (and (consp alist)
+              (or (not (consp (car alist)))
+                  (equal (car (car alist)) key)))
+    (setq alist (cdr alist)))
+  (if (consp alist)
+      (let ((prev alist)
+            (tail (cdr alist)))
+        (while (consp tail)
+          (if (and (consp (car alist))
+                   (equal (car (car tail)) key))
+              ;; `(setcdr CELL NEWCDR)' returns NEWCDR.
+              (setq tail (setcdr prev (cdr tail)))
+            (setq prev (cdr prev)
+                  tail (cdr tail))))))
+  alist)
 
-;; XEmacs 19.13 and later: (remrassq VALUE LIST)
-(defun-maybe remrassq (value list)
-  "Delete by side effect any elements of LIST whose cdr is `eq' to VALUE.
-The modified LIST is returned.  If the first member of LIST has a car
-that is `eq' to VALUE, there is no way to remove it by side effect;
-therefore, write `(setq foo (remrassq value foo))' to be sure of changing
+;; XEmacs 19.13 and later: (remassq KEY ALIST)
+(defun-maybe remassq (key alist)
+  "Delete by side effect any elements of ALIST whose car is `eq' to KEY.
+The modified ALIST is returned.  If the first member of ALIST has a car
+that is `eq' to KEY, there is no way to remove it by side effect;
+therefore, write `(setq foo (remassq key foo))' to be sure of changing
 the value of `foo'."
-  (if (setq value (rassq value list))
-      (delq value list)
-    list))
+  (while (and (consp alist)
+              (or (not (consp (car alist)))
+                  (eq (car (car alist)) key)))
+    (setq alist (cdr alist)))
+  (if (consp alist)
+      (let ((prev alist)
+            (tail (cdr alist)))
+        (while (consp tail)
+          (if (and (consp (car tail))
+                   (eq (car (car tail)) key))
+              ;; `(setcdr CELL NEWCDR)' returns NEWCDR.
+              (setq tail (setcdr prev (cdr tail)))
+            (setq prev (cdr prev)
+                  tail (cdr tail))))))
+  alist)
 
-;; XEmacs 19.13 and later: (remrassoc VALUE LIST)
-(defun-maybe remrassoc (value list)
-  "Delete by side effect any elements of LIST whose cdr is `equal' to VALUE.
-The modified LIST is returned.  If the first member of LIST has a car
+;; XEmacs 19.13 and later: (remrassoc VALUE ALIST)
+(defun-maybe remrassoc (value alist)
+  "Delete by side effect any elements of ALIST whose cdr is `equal' to VALUE.
+The modified ALIST is returned.  If the first member of ALIST has a car
 that is `equal' to VALUE, there is no way to remove it by side effect;
 therefore, write `(setq foo (remrassoc value foo))' to be sure of changing
 the value of `foo'."
-  (if (setq value (rassoc value list))
-      (delq value list)
-    list))
+  (while (and (consp alist)
+              (or (not (consp (car alist)))
+                  (equal (cdr (car alist)) value)))
+    (setq alist (cdr alist)))
+  (if (consp alist)
+      (let ((prev alist)
+            (tail (cdr alist)))
+        (while (consp tail)
+          (if (and (consp (car tail))
+                   (equal (cdr (car tail)) value))
+              ;; `(setcdr CELL NEWCDR)' returns NEWCDR.
+              (setq tail (setcdr prev (cdr tail)))
+            (setq prev (cdr prev)
+                  tail (cdr tail))))))
+  alist)
+
+;; XEmacs 19.13 and later: (remrassq VALUE ALIST)
+(defun-maybe remrassq (value alist)
+  "Delete by side effect any elements of ALIST whose cdr is `eq' to VALUE.
+The modified ALIST is returned.  If the first member of ALIST has a car
+that is `eq' to VALUE, there is no way to remove it by side effect;
+therefore, write `(setq foo (remrassq value foo))' to be sure of changing
+the value of `foo'."
+  (while (and (consp alist)
+              (or (not (consp (car alist)))
+                  (eq (cdr (car alist)) value)))
+    (setq alist (cdr alist)))
+  (if (consp alist)
+      (let ((prev alist)
+            (tail (cdr alist)))
+        (while (consp tail)
+          (if (and (consp (car tail))
+                   (eq (cdr (car tail)) value))
+              ;; `(setcdr CELL NEWCDR)' returns NEWCDR.
+              (setq tail (setcdr prev (cdr tail)))
+            (setq prev (cdr prev)
+                  tail (cdr tail))))))
+  alist)
 
 ;;; Define `functionp' here because "localhook" uses it.
 
@@ -1112,8 +1160,7 @@ which is made by replacing the part of STRING that was matched."
 		    (buffer-string)))
 	       (si:replace-match newtext fixedcase literal)))))))))
 
-;; Emacs 20: (format-time-string)
-;; The the third optional argument universal is yet to be implemented.
+;; Emacs 20: (format-time-string FORMAT &optional TIME UNIVERSAL)
 ;; Those format constructs are yet to be implemented.
 ;;   %c, %C, %j, %U, %W, %x, %X
 ;; Not fully compatible especially when invalid format is specified.
@@ -1187,7 +1234,6 @@ For example, to produce full ISO 8601 format, use \"%Y-%m-%dT%T%z\".
 
 Compatibility Note.
 
-The the third optional argument universal is yet to be implemented.
 Those format constructs are yet to be implemented.
   %c, %C, %j, %U, %W, %x, %X
 Not fully compatible especially when invalid format is specified."
@@ -1202,9 +1248,24 @@ Not fully compatible especially when invalid format is specified."
 	  field-result
 	  pad-left change-case
 	  (paren-level 0)
-	  hour
-	  (time-string (current-time-string time)))
-      (setq hour (string-to-int (substring time-string 11 13)))
+	  hour ms ls
+	  (tz (car (current-time-zone)))
+	  time-string)
+      (if universal
+	  (progn
+	    (or time
+		(setq time (current-time)))
+	    (setq ms (car time)
+		  ls (- (nth 1 time) tz))
+	    (cond ((< ls 0)
+		   (setq ms (1- ms)
+			 ls (+ ls 65536)))
+		  ((>= ls 65536)
+		   (setq ms (1+ ms)
+			 ls (- ls 65536))))
+	    (setq time (append (list ms ls) (nth 2 time)))))
+      (setq time-string (current-time-string time)
+	    hour (string-to-int (substring time-string 11 13)))
       (while (< ind fmt-len)
 	(setq cur-char (aref format ind))
 	(setq
@@ -1253,7 +1314,7 @@ Not fully compatible especially when invalid format is specified."
 		(cond
 		 ((eq cur-char ?%)
 		  "%")
-		 ;; the abbreviated name of the day of week.		 
+		 ;; the abbreviated name of the day of week.
 		 ((eq cur-char ?a)
 		  (substring time-string 0 3))
 		 ;; the full name of the day of week
@@ -1274,7 +1335,7 @@ Not fully compatible especially when invalid format is specified."
 		 ((eq cur-char ?C)
 		  "")
 		 ;; the day of month, zero-padded
-		 ((eq cur-char ?d)	
+		 ((eq cur-char ?d)
 		  (format "%02d" (string-to-int (substring time-string 8 10))))
 		 ;; a synonym for `%m/%d/%y'
 		 ((eq cur-char ?D)
@@ -1342,7 +1403,7 @@ Not fully compatible especially when invalid format is specified."
 			  (substring time-string 11 13)
 			  (substring time-string 14 16)
 			  (substring time-string 17 19)))
-		 ;; the week of the year (01-52), assuming that weeks 
+		 ;; the week of the year (01-52), assuming that weeks
 		 ;; start on Sunday (yet to come)
 		 ((eq cur-char ?U)
 		  "")
@@ -1368,8 +1429,18 @@ Not fully compatible especially when invalid format is specified."
 		  (substring time-string -4))
 		 ;; the time zone abbreviation
 		 ((eq cur-char ?Z)
-		  (setq change-case (not change-case))
-		  (downcase (cadr (current-time-zone))))
+		  (if universal
+		      "UTC"
+		    (setq change-case (not change-case))
+		    (downcase (cadr (current-time-zone)))))
+		 ((eq cur-char ?z)
+		  (if universal
+		      "+0000"
+		    (if (< tz 0)
+			(format "-%02d%02d"
+				(/ (- tz) 3600) (/ (% (- tz) 3600) 60))
+		      (format "+%02d%02d"
+			      (/ tz 3600) (/ (% tz 3600) 60)))))
 		 (t
 		  (concat
 		   "%"
@@ -1411,6 +1482,47 @@ Not fully compatible especially when invalid format is specified."
   (setq current-load-list (cons 'format-time-string current-load-list))
   (put 'format-time-string 'defun-maybe t))))
 
+;; Emacs 19.29-19.34/XEmacs: `format-time-string' neither supports the
+;; format string "%z" nor the third argument `universal'.
+(unless (string-match "\\`[\\-\\+][0-9]+\\'"
+		      (format-time-string "%z" (current-time)))
+  (defadvice format-time-string
+    (before support-timezone-in-numeric-form-and-3rd-arg
+	    (format-string &optional time universal) activate compile)
+    "Advice to support the construct `%z' and the third argument `universal'."
+    (let ((tz (car (current-time-zone)))
+	  case-fold-search ms ls)
+      (while (string-match "\\(\\(\\`\\|[^%]\\)\\(%%\\)*\\)%z" format-string)
+	(setq format-string
+	      (concat (substring format-string 0 (match-end 1))
+		      (if universal
+			  "+0000"
+			(if (< tz 0)
+			    (format "-%02d%02d"
+				    (/ (- tz) 3600) (/ (% (- tz) 3600) 60))
+			  (format "+%02d%02d"
+				  (/ tz 3600) (/ (% tz 3600) 60))))
+		      (substring format-string (match-end 0)))))
+      (if universal
+	  (progn
+	    (while (string-match "\\(\\(\\`\\|[^%]\\)\\(%%\\)*\\)%Z"
+				 format-string)
+	      (setq format-string
+		    (concat (substring format-string 0 (match-end 1))
+			    "UTC"
+			    (substring format-string (match-end 0)))))
+	    (or time
+		(setq time (current-time)))
+	    (setq ms (car time)
+		  ls (- (nth 1 time) tz))
+	    (cond ((< ls 0)
+		   (setq ms (1- ms)
+			 ls (+ ls 65536)))
+		  ((>= ls 65536)
+		   (setq ms (1+ ms)
+			 ls (- ls 65536))))
+	    (setq time (append (list ms ls) (nth 2 time))))))))
+
 ;; Emacs 20.1/XEmacs 20.3(?) and later: (split-string STRING &optional PATTERN)
 ;; Here is a XEmacs version.
 (defun-maybe split-string (string &optional pattern)
