@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-19.el - cmail functions for emacs 19
 ;;; 
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Fri May 27 13:33:49 JST 1994
 ;;;
 ;;;  Maintener:     Toshihiko Ueki <toshi@he.kobelcosys.co.jp>
@@ -28,10 +28,15 @@
       (hilit-set-mode-patterns 'cmail-summary-mode
 			       '(("^ +[0-9]+ .*" nil summary-seen)
 				 ("^ +[0-9]+R.*" nil msg-from)
+				 ("^ +[0-9]+F.*" nil msg-from)
+				 ("^ +[0-9]+>.*" nil msg-from)
 				 ("^ +[0-9]+E.*" nil summary-Xed)
 				 ("^ +[0-9]+U.*" nil summary-new)
 				 ("^ +[0-9]+D.*" nil summary-deleted)
 				 ("^ +[0-9]+H.*" nil msg-quote)
+				 ("^ +[0-9]+B.*" nil msg-quote)
+				 ("^ +[0-9]+b.*" nil msg-quote)
+				 ("^ +[0-9]+!.*" nil summary-deleted)
 				 ("^\+.*" nil summary-current)))
 
       (defun cmail-rehighlight-line ()
