@@ -6,7 +6,7 @@
 ;;; Author: Toshihiko Ueki <toshi@he.kobelcosys.co.jp>
 ;;; Created:       1997/04/20
 ;;; Last modified: 2000/06/14
-;;; Version: $Revision: 1.5 $
+;;; Version: $Revision: 1.3 $
 
 ;;; This file is not part of mailcrypt.
 ;;;
@@ -40,7 +40,7 @@
 ;;; @ version definition
 ;;;
 
-(defconst cmail-crypt-version "$Revision: 1.5 $" "*cmail-crypt version")
+(defconst cmail-crypt-version "$Revision: 1.3 $" "*cmail-crypt version")
 
 ;;; @ variables
 ;;;
@@ -254,11 +254,8 @@
 		      (beginning-of-buffer))
 		    (save-excursion
 		      (cmail-crypt-change-pgp-version-temporarily tmpver))))
-	      (defalias 'tmp-err 'error)
-	      (defalias 'error 'message)
-	      (unwind-protect
-		  (funcall func)
-		(defalias 'error 'tmp-err))
+	      (flet ((error (&rest args) (apply 'message args)))
+		(funcall func))
 	      (set-buffer-modified-p nil)
 	      (cmail-go-summary))))))
 
