@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-e19.el - cmail functions for emacs 19 or later
 ;;; 
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;           toshi@he.kobelcosys.co.jp
 ;;;  created at: Wed Aug 20 14:00:00 JST 1997
 ;;;  last modified at: Wed May 31 12:00:00 JST 2000
@@ -25,12 +25,13 @@
 ;;
 ;; mail mode menu
 ;;
+(defvar cmail-mail-menu-bar nil)
 (defadvice cmail-derived-mail-mode-setter
   (before cmail-derived-mail-mode-setter-emacs-19 activate)
   ;; ... keymaps are merged ok, but menus are not quite so.
   ;; ... I will copy it here.
-  (defvar cmail-mail-menu-bar
-    (copy-keymap (lookup-key cmail-mail-mode-map [menu-bar mail])))
+  (setq cmail-mail-menu-bar
+        (copy-keymap (lookup-key cmail-mail-mode-map [menu-bar mail])))
   (define-key cmail-mail-mode-map [menu-bar cmail]
     (cons "Cmail" cmail-mail-menu-bar)))
 
