@@ -113,9 +113,10 @@
 (defvar *cmail-folders-beginning-point nil)
 (defvar *cmail-folders-updated-folders-list '())
 (defvar *cmail-folders-removed-folders-list '())
+(defvar *cmail-folders-modified nil)
 (defvar *cmail-folders-cache-modified nil)
 (defvar *cmail-folders-system-folders-list
-  (list cmail-incoming-mails-folder *cmail-reorder-folder))
+  (list cmail-incoming-mails-folder))
 (defvar *cmail-folders-set-overlay 'ignore)
 (defvar cmail-folders-mail-count-column 6) ;;; $B8e$G(Bcmail-vars.el$B$"$?$j$K0\F0(B
 (defvar *cmail-folders-folder-format
@@ -204,7 +205,13 @@
 
 ;;; $B2>A[%U%)%k%@$+(B?
 (defsubst cmail-folders-is-virtual-folder (folder)
-  (= (elt folder 0) ?/))
+  (if folder
+      (or (eq (aref folder 0) ?/)
+	  (if (and
+	       (boundp 'cmail-virtual-folder-check-function)
+	       (functionp cmail-virtual-folder-check-function))
+	      (funcall
+	       cmail-virtual-folder-check-function folder)))))
 
 ;;; $B%U%)%k%@$N%G%#%l%/%H%jItA4BN$rJV$9(B ($B%H%C%W3,AX0J30$O=*C<$K(B`/'$B$,IU$/(B)
 ;;; folder$B$N=*C<$,(B`/'$B$J$i$P(Bfolder$BA4BN$rJV$9(B
@@ -232,10 +239,27 @@
 (defsubst cmail-folders-get-sub-folder (folder)
   (substring folder *cmail-folders-current-dir-length))
 
+(defsubst cmail-folders-nickname (folder)
+  (if cmail-folders-nickname-display
+      (let ((nick (rassoc folder (cmail-folders-nickname-alist))))
+	(if nick
+	    (concat *cmail-folders-current-dir (concat (car nick) "*"))
+	  folder))
+    folder))
+
+(defun cmail-folders-nickname-realname (nick)
+  (let (fname f)
+    (if (and cmail-folders-nickname-display
+	     (string-match "\\*$" nick)
+	     (setq fname (substring nick 0 -1))
+	     (setq f (assoc fname (cmail-folders-nickname-alist))))
+	(cdr f)
+      nick)))
+
 ;;; $B;XDj$N%U%)%k%@$K%^%C%A$9$k%Q%?!<%s$rJV$9(B
 (defsubst cmail-folders-get-pattern (folder)
   (concat *cmail-folders-pattern
-	  "\\(" (regexp-quote (cmail-folders-get-sub-folder folder)) "\\)$"))
+	  "\\(" (regexp-quote (cmail-folders-get-sub-folder (cmail-folders-nickname folder))) "\\)$"))
 
 ;;; $B%+%l%s%H%G%#%l%/%H%j$NJQ99(B
 (defsubst cmail-folders-chdir (dir)
@@ -313,7 +337,6 @@
 ;;; $B%b!<%I@_Dj(B
 (defun cmail-folders-mode ()
   "cmail$B$N%U%)%k%@$rA*Br$9$k(B.
-
 $B%-!<%P%$%s%I$O0J2<$NDL$j(B,
 \\{cmail-folders-mode-map}"
   (interactive)
@@ -334,10 +357,11 @@
 ;;; $B2<5-$N$h$&$J%j%9%H$rJV$9(B
 ;;; "DIR1/DIR2/"       -> ("DIR1/")
 ;;; "DIR1/DIR2/FOLDER" -> ("DIR1/" "DIR1/DIR2/")
-;;; folder$B$,2>A[%U%)%k%@$J$i(Bnil$B$rJV$9(B
+;;; folder$B$,(B"/"$B$G;O$^$k2>A[%U%)%k%@$J$i(Bnil$B$rJV$9(B
 (defun cmail-folders-get-path-list (folder)
-  (if (cmail-folders-is-virtual-folder folder)
-      nil
+ (if (and (cmail-folders-is-virtual-folder folder)
+	  (= (elt folder 0) ?/))
+     nil
     (let ((index (- (length folder) 2)) path-list)
       (while (natnump index)
 	(if (= (elt folder index) ?/)
@@ -415,7 +439,8 @@
   (push entry new-entries)
   (mapcar
    '(lambda (dir)
-      (if (null (assoc dir (cmail-folders-cache-alist)))
+      (if (and (null (assoc dir (cmail-folders-cache-alist)))
+	       (null (assoc dir new-entries)))
 	  (push (cmail-folders-create-ce-directory dir) new-entries)))
    (cmail-folders-get-path-list (car entry))))
 
@@ -559,10 +584,12 @@
 	     (depth (cmail-folders-get-ce-depth entry))
 	     (unum (cmail-folders-get-ce-unread-mail-num entry)))
 	(cond
-	 ;; $BDL>o$N%U%!%$%k5Z$S2>A[%U%)%k%@$OL5;k(B
-	 ((or (null unum) (cmail-folders-is-virtual-folder folder))
+	 ;; $BDL>o$N%U%!%$%k$H(B"/"$B$G$O$8$^$k2>A[%U%)%k%@$OL5;k(B
+	 ((or (null unum)
+	      (and (cmail-folders-is-virtual-folder folder)
+		   (= (elt folder 0) ?/)))
 	  nil)
-	 ;; $B%G%#%l%/%H%j(B
+	 ;; $B$=$l0J30$N2>A[%U%)%k%@$^$?$O%G%#%l%/%H%j(B
 	 ((eq (cmail-folders-get-ce-status entry) 'directory)
 	  (cmail-folders-set-ce-folder-num entry 0)
 	  (cmail-folders-set-ce-unread-mail-num entry 0)
@@ -654,18 +681,36 @@
 	   ((listp (cmail-folders-get-ce-status entry)) ":")
 	   ((member (car entry) cmail-folders-unfold-dirs) "-")
 	   (t "+"))
-	  (cmail-folders-get-sub-folder (car entry))))
+	  (let ((folder (car entry)) f)
+	    (if cmail-folders-nickname-display
+		(progn
+		  (if (setq f (rassoc folder (cmail-folders-nickname-alist)))
+		      (concat (car f) "*")
+		    (cmail-folders-get-sub-folder folder)))
+	      (cmail-folders-get-sub-folder folder)))))
 
-;;; $BI=<($7$F$b$h$$%U%)%k%@$+(B?
+;;; $B@^$j$?$?$^$:$KI=<($7$F$b$h$$%U%)%k%@(B/$B%G%#%l%/%H%j$+(B? 
 (defun cmail-folders-output-p (folder)
   (and folder
+       ;; extract the part of the folder string relative to current
+       ;; directory.
        (string-match *cmail-folders-current-dir-pattern folder)
-       (or (string-match "^[^/]+/?$" (substring folder (match-end 1)))
-	   (member (cmail-folders-get-dir-head2 folder)
-		   cmail-folders-unfold-dirs)
-	   (member folder cmail-folders-unfold-dirs)
-	   (and (cmail-folders-is-virtual-folder folder)
-		(zerop *cmail-folders-current-dir-length)))))
+       (or 
+	;; display if the folder is NOT in the subdirectory, either
+	;; plain folder or directory name.
+	(string-match "^[^/]+/?$" (substring folder (match-end 1)))
+	;; display if the folder dir is NOT in the dir to be folded
+	(member (cmail-folders-get-dir-head2 folder)
+		cmail-folders-unfold-dirs)
+	;; display if the folder dir is NOT the dir to be folded
+	(member folder cmail-folders-unfold-dirs)
+	;; display if the folder dir is top dir and is virtual folder
+	;; and is one that start with "/" character.
+	(and (cmail-folders-is-virtual-folder folder)
+	     (zerop *cmail-folders-current-dir-length)
+	     (= (elt folder 0) ?/))
+	(and cmail-folders-nickname-display
+	     (rassoc folder (cmail-folders-nickname-alist))))))
 
 ;;; $B%(%s%H%j$KBP1~$9$kI=<(MQJ8;zNs$rJV$9!#(B
 ;;; $B=PNO$7$J$$%(%s%H%j$KBP$7$F$O(Bnil$B$rJV$9(B
@@ -690,6 +735,15 @@
 	(skip-folders-list (append cmail-folders-list-top
 				   cmail-folders-list-bottom nil))
 	cursor)
+    (if cmail-folders-nickname-display
+	(progn
+	  (setq cursor (cmail-folders-nickname-realnames))
+	  (while cursor
+	    (let* ((folder (car cursor))
+		   (entry (assoc folder cache-alist)))
+	      (cmail-folders-output-line entry)
+	      (setq skip-folders-list (cons folder skip-folders-list))
+	      (setq cursor (cdr cursor))))))
     (setq cursor cmail-folders-list-top)
     (while cursor
       (let ((entry (assoc (car cursor) cache-alist)))
@@ -757,6 +811,9 @@
 	       ((eq type 'top)
 		(eq (cmail-folders-<-on-list (match-string 1) sub-folder
 					     sub-list-top) t))
+	       ((eq type 'nick)
+		(eq (cmail-folders-<-on-list (match-string 1) sub-folder
+					     nick-list) t))
 	       (t ;;; (eq type 'bottom)
 		(cmail-folders-<-on-list (match-string 1) sub-folder
 					 sub-list-bottom)))))
@@ -815,6 +872,8 @@
 (defun cmail-folders-modify-lines (func list)
   (let ((top-list (cmail-folders-pickup-folders 
 		   cmail-folders-list-top list))
+	(nick-list (cmail-folders-pickup-folders
+		    (cmail-folders-nickname-realnames) list))
 	(bottom-list (cmail-folders-pickup-folders
 		      cmail-folders-list-bottom list))
 	(sub-list-top (cmail-folders-get-sub-folders-list 
@@ -824,6 +883,8 @@
 	folder-list type)
     (setq folder-list (nconc (list 'top)
 			     top-list
+			     (list 'nick)
+			     nick-list
 			     (list 'middle)
 			     (sort (delq nil list) 'string<)
 			     (list 'bottom)
@@ -1025,7 +1086,10 @@
 			 (cmail-folders-get-cache-entry-from-buffer folder)
 		       (cmail-folders-get-cache-entry-from-file folder))))
       (cmail-folders-push-new-entry
-       (cmail-folders-get-cache-entry-from-file folder))
+       (if (and (cmail-folders-is-virtual-folder folder)
+		(member folder *cmail-opened-folders-list))
+	   (cmail-folders-get-cache-entry-from-buffer folder)
+	 (cmail-folders-get-cache-entry-from-file folder)))
       (cmail-folders-add-new-entries new-entries))))
 
 ;;; folder$B$KBP1~$9$k%-%c%C%7%e%(%s%H%j$r:o=|$9$k(B
@@ -1117,6 +1181,8 @@
     (and (setq buf (find-buffer-visiting *cmail-folders-cache))
 	 (kill-buffer buf))
     (and (setq buf (get-buffer *cmail-folders-tmp-buffer))
+	 (kill-buffer buf))
+    (and (setq buf (get-buffer *cmail-folders-nickname-buffer))
 	 (kill-buffer buf)))
   (setq *cmail-folders-cache-modified nil)
   (setq *cmail-folders-cache-alist nil)
@@ -1203,7 +1269,12 @@
 	    (goto-char (match-beginning 1))
 	    (if (string= sub-folder "../")
 		(cmail-folders-get-parent-dir)
-	      (concat *cmail-folders-current-dir sub-folder)))))))
+	      (if (and cmail-folders-nickname-display
+		       (string-match "\\*$" sub-folder)
+		       (setq sub-folder (substring sub-folder 0 -1)))
+		  (let ((f (assoc sub-folder (cmail-folders-nickname-alist))))
+		    (and f (cdr f)))
+		(concat *cmail-folders-current-dir sub-folder))))))))
 
 ;;; $B%G%#%l%/%H%j$+H]$+(B
 (defun cmail-folders-is-dir (folder)
@@ -1234,7 +1305,7 @@
 ;;; $B%U%)%k%@$K(Bvisit$B$9$k(B
 (defun cmail-folders-visit-sub-folder (folder)
   (let (*cmail-prev-folder)
-    (set-buffer *cmail-summary-buffer)
+    (set-buffer (get-buffer-create *cmail-summary-buffer))
     (cmail-visit-folder folder t)))
 
 (defun cmail-folders-visit-folder-without-thread ()
@@ -1347,24 +1418,42 @@
 (defun cmail-folders-get-newmail ()
   "$B?7Ce%a!<%k$r<hF@$9$k(B."
   (interactive)
-  (cond
-   ((cmail-get-spooled-mail)
-    (setq cmail-current-folder cmail-inbox-folder)
+  (let (updated-folders updated-p)
+    (if (and (boundp 'cmail-folders-update-function)
+	     (functionp cmail-folders-update-function))
+	(progn
+	  (setq updated-folders (funcall cmail-folders-update-function))
+	  (if updated-folders
+	      (progn
+		(setq updated-p t)
+		(if cmail-archive-on-get
+		    (progn
+		      (while updated-folders
+			(cmail-make-summary (car updated-folders))
+			(cmail-auto-archive nil t)
+			(setq updated-folders (cdr updated-folders)))
+		      (cmail-folders nil t))
+		  (cmail-folders-redisplay))))))
     (cond
-     (cmail-archive-on-get
-      (cmail-make-summary cmail-inbox-folder)
-      (cmail-auto-archive)
-      (cmail-folders nil t))
-     (cmail-use-folders-mode-first
-      (cmail-folders nil t)
-      (cmail-make-summary cmail-inbox-folder)
-      (cmail-folders-visit-sub-folder cmail-inbox-folder))
-     (t
-      (cmail-folders nil t))))
-   (t
-    (or cmail-folders-display-all-folders
+     ((cmail-get-spooled-mail)
+      (setq updated-p t)
+      (setq cmail-current-folder cmail-inbox-folder)
+      (cond
+       (cmail-archive-on-get
+	(cmail-make-summary cmail-inbox-folder)
+	(cmail-auto-archive)
 	(cmail-folders nil t))
-    (cmail-message-resource 'get-spooled-mail-9))))
+       (cmail-use-folders-mode-first
+	(cmail-folders nil t)
+	(cmail-make-summary cmail-inbox-folder)
+	(cmail-folders-visit-sub-folder cmail-inbox-folder))
+       (t
+	(cmail-folders nil t))))
+     (t
+      (or cmail-folders-display-all-folders
+	  (cmail-folders nil t))))
+    (if (not updated-p)
+	(cmail-message-resource 'get-spooled-mail-9))))
 
 ;;; interactive$B$N$?$a$N%Q%i%a!<%?$rJV$9(B
 (defun cmail-folders-get-interactive-param (prompt emes)
@@ -1483,6 +1572,9 @@
 	       (and (eq cmail-compressed-folders 'jam-zcat)
 		    (string-match "\\.gz$\\|\\.Z$" folder)
 		    (setq folder (substring folder 0 (match-beginning 0))))
+	       (and cmail-folders-nickname-display
+		    (let ((f (rassoc folder *cmail-folders-nickname-alist)))
+		      (if f (setq folder (concat (car f) "*")))))
 	       (push (list folder) folders-list))))
      (cmail-folders-cache-alist))
     (nreverse folders-list)))
@@ -1497,9 +1589,48 @@
        '(lambda (folder)
 	  (if (equal folder cmail-inbox-folder)
 	      (cmail-save-folder folder)
-	    (cmail-dismiss-folder folder)
-	    (cmail-message-resource1 'folders-close-all-folders-1 folder)))
+	    (unless (cmail-folders-is-virtual-folder folder)
+	      (cmail-dismiss-folder folder)
+	      (cmail-message-resource1 'folders-close-all-folders-1 folder))))
        *cmail-opened-folders-list))
     (cmail-message-resource 'folders-close-all-folders-2)))
 
+(defvar cmail-folders-nickname-display nil)
+(defvar *cmail-folders-nickname-alist '())
+(defun cmail-folders-nickname-realnames ()
+  (let ((cursor (cmail-folders-nickname-alist)) realnames)
+    (while cursor
+      (setq realnames (cons (cdr (car cursor)) realnames))
+      (setq cursor (cdr cursor)))
+    realnames))
+
+(defvar *cmail-folders-nicknames
+  (expand-file-name ".cmail-folder-nicknames" cmail-path))
+(defvar *cmail-folders-nicknames-age nil)
+(defconst *cmail-folders-nickname-buffer " *cmail-folders-nicknames*")
+
+(defun cmail-folders-load-nicknames ()
+  (save-excursion
+    (set-buffer (find-file-noselect *cmail-folders-nicknames))
+    (auto-save-mode nil)
+    (rename-buffer *cmail-folders-nickname-buffer)
+    (let ((buffer-read-only t)
+	  (inhibit-read-only t)
+	  nick folder alist)
+      (goto-char (point-min))
+      (while (re-search-forward "^\\([^ \t]*\\)[ \t]+\\(.*\\)" nil t)
+	(setq folder (buffer-substring (match-beginning 1) (match-end 1)))
+	(setq nick (buffer-substring (match-beginning 2) (match-end 2)))
+	(setq alist (cons (cons folder nick) alist)))
+      (setq inhibit-read-only nil)
+      alist)))
+
+(defun cmail-folders-nickname-alist ()
+  (if (and *cmail-folders-nickname-alist
+	   *cmail-folders-nicknames-age
+	   (cmail-older (nth 5 (file-attributes *cmail-folders-nicknames))
+			*cmail-folders-nicknames-age))
+      *cmail-folders-nickname-alist
+    (setq *cmail-folders-nickname-alist (cmail-folders-load-nicknames))))
+
 (provide 'cmail-folders)
