@@ -38,6 +38,8 @@
 (make-face 'cmail-summary-mode-unread-face)
 (make-face 'cmail-summary-mode-delete-face)
 (make-face 'cmail-summary-mode-hold-face)
+(make-face 'cmail-summary-mode-bookmark-face)
+(make-face 'cmail-summary-mode-moreflags-face)
 (make-face 'cmail-summary-mode-marked-face)
 (make-face 'cmail-summary-mode-current-face)
 (make-face 'cmail-folders-mode-dir-face)
@@ -196,10 +198,15 @@
 		       ((eq (elt line 0) ?+) 'cmail-summary-mode-current-face)
 		       ((eq mark ? ) 'cmail-summary-mode-seen-face)
 		       ((eq mark ?R) 'cmail-summary-mode-reply-face)
+		       ((eq mark ?F) 'cmail-summary-mode-reply-face)
+		       ((eq mark ?>) 'cmail-summary-mode-reply-face)
 		       ((eq mark ?E) 'cmail-summary-mode-edited-face)
 		       ((eq mark ?U) 'cmail-summary-mode-unread-face)
 		       ((eq mark ?D) 'cmail-summary-mode-delete-face)
 		       ((eq mark ?H) 'cmail-summary-mode-hold-face)
+		       ((eq mark ?B) 'cmail-summary-mode-bookmark-face)
+		       ((eq mark ?b) 'cmail-summary-mode-bookmark-face)
+		       ((eq mark ?!) 'cmail-summary-mode-moreflags-face)
 		       ((eq mark ?^) 'cmail-summary-mode-marked-face)))))
       )))
 
