@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-misc.el - utility functions for cmail
 ;;;
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Mon Jul 12 10:44:23 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto
@@ -150,11 +150,126 @@
 (defun cmail-skip-From_ ()
   (while (looking-at "^From ") (forward-line)))
   
-(defun cmail-set-mail-status (status)
+(defconst *cmail-mail-status-mark-assoc
+  '(("^Active\\(,.*\\|$\\)" " " "A")
+    ("^Unread\\(,.*\\|$\\)" "U" "U")
+    ("^Edited\\(,.*\\|$\\)" "E" "E")
+    ("^Deleted\\(,.*\\|$\\)" "D" "D")
+    ("^Hold\\(,.*\\|$\\)" "H" "H")
+    ("^Replied$" "R" "R")
+    ("^Forwarded$" "F" "F")
+    ("^Bookmarked$" "B" "B")
+    ("^Replied,Forwarded$" ">" ">")
+    ("^Replied,Bookmarked$" "b" "b")
+    ("^Replied,Forwarded,Bookmarked$" "b" "b")))
+
+(defun cmail-get-mark-symbol (status-string)
+  (let ((markstr-db *cmail-mail-status-mark-assoc))
+    (catch 'cmail-get-mark-symbol-return
+      (while markstr-db
+	(if (string-match (car (car markstr-db)) status-string)
+	    (throw 'cmail-get-mark-symbol-return
+		   (cdr (car markstr-db))))
+	(setq markstr-db (cdr markstr-db)))
+      '("!" "!"))))
+
+(defun cmail-set-mail-status (add-status &optional del-status)
   (cmail-skip-From_)
-  (if (re-search-forward "^X-cmail-status: \\(.*$\\)" (cmail-head-max) t)
-      (replace-match (format "X-cmail-status: %s" status) t t)
-    (insert-string (format "X-cmail-status: %s\n" status))))
+  ;; For backward compatibility, allow string for add-status
+  (let (status status-string new-status mod-p)
+    (if (stringp add-status) (setq add-status (list add-status)))
+    (if (re-search-forward "^X-cmail-status:[ \t]*\\(.*$\\)"
+			   (cmail-head-max) t)
+	(progn
+	  (setq status-string (match-string 1))
+	  (setq status (cmail-set-mail-status1 status-string
+					       add-status del-status))
+	  ;; nil is returned if no change.
+	  (if status
+	      (progn
+		(setq new-status (mapconcat '(lambda (x) x) status ","))
+		(if (not (string= new-status status-string))
+		    (progn
+		      (setq mod-p t)
+		      (replace-match
+		       (format "X-cmail-status: %s" new-status t t)))))
+	    (setq new-status status-string)))
+      (setq status (cmail-sort-mail-status add-status))
+      (setq new-status (mapconcat '(lambda (x) x) status ","))
+      (insert-string (format "X-cmail-status: %s\n" new-status))
+      (setq mod-p t))
+    ;; imap4 update, only when status is updated
+    (if (and mod-p (boundp 'cmail-message-status-function)
+	     (functionp cmail-message-status-function))
+	(funcall cmail-message-status-function new-status))
+    (cmail-get-mark-symbol new-status)))
+
+(defun cmail-set-mail-status-if-missing (add-status)
+  (cmail-skip-From_)
+  (let ((beg (point)) status-string)
+    ;; if status is already present, do nothing.
+    (if (re-search-forward "^X-cmail-status:[ \t]*\\(.*$\\)"
+			   (cmail-head-max) t)
+	nil
+      ;; otherwise, insert status string.
+      (setq status-string
+	    (mapconcat
+	     '(lambda (x) x) (cmail-sort-mail-status add-status) ","))
+      (goto-char beg)
+      (insert-string (format "X-cmail-status: %s\n" status-string)))))
+
+(defun cmail-set-mail-status1 (status-string add-status del-status)
+  (let ((cur-status status-string) status-list status mod-p)
+    (save-match-data
+      (while (string-match "\\([^,]+\\)\\(,.*\\|\\)$" cur-status)
+	(setq status (match-string 1 cur-status))
+	(if (member status del-status)
+	    (setq mod-p t)
+	  (setq status-list (cons status status-list)))
+	(setq cur-status (match-string 2 cur-status))))
+    (while add-status
+      (if (not (member (car add-status) status-list))
+	  (progn
+	    (setq mod-p t)
+	    (setq status-list (cons (car add-status) status-list))))
+      (setq add-status (cdr add-status)))
+    (and mod-p (cmail-sort-mail-status status-list))))
+
+(defconst *cmail-sort-mail-status-order
+  '("Active" "Deleted" "Hold" "Unread"
+    "Edited" "Replied" "Forwarded" "Bookmarked"))
+
+(defun cmail-sort-mail-status (status-list)
+  (let ((order *cmail-sort-mail-status-order) res)
+    (while order
+      (if (member (car order) status-list)
+	  (progn
+	    (setq res (cons (car order) res))
+	    (setq status-list (delete (car order) status-list))))
+      (setq order (cdr order)))
+    ;; unknown status will be discarded for now.
+    (setq res (nreverse res))
+    (if (and res (equal (car res) "Active"))
+	(or (cdr res) res)
+      res)))
+
+(defun cmail-mail-status-p (status)
+  (let ((status-string (cmail-get-field-values "X-cmail-status")))
+    (string-match (concat "\\(,\\|^\\)" (regexp-quote status)
+			  "\\(,\\|$\\)") status-string)))
+
+(defun cmail-delete-field (field-regexp)
+  "FIELD-REGEXP$B$KEv$F$O$^$k%X%C%@$N%U%#!<%k%I$r:o=|$9$k(B. "
+  (let ((case-fold-search t) max beg)
+    (if (string= (substring field-regexp 0 1) "^")
+	nil
+      (setq field-regexp (concat "^" field-regexp ":[ \t]*")))
+    (setq max (cmail-head-max))
+    (while (re-search-forward field-regexp max t)
+      (setq beg (match-beginning 0))
+      (forward-line 1)
+      (while (looking-at "^[ \t]+") (forward-line 1))
+      (delete-region beg (point)))))
 
 (defun cmail-get-field-values (field-regexp &optional sep sep2)
   "FIELD-REGEXP$B$KEv$F$O$^$k%X%C%@$N%U%#!<%k%ICM$rJV$9(B."
@@ -198,7 +313,23 @@
     (cmail-get-field-values field-regexp sep sep2)))
 (defalias 'cmail-get-field-values2 'cmail-get-field-values-in-mail-buffer)
 
-(defun cmail-trans-date-form (date)
+(defun cmail-set-field-value (field-str value-str &optional appendp)
+  "FIELD-STR $B%X%C%@$N%U%#!<%k%I$r(B VALUE-STR $B$K%;%C%H$9$k(B. APPENDP $B$,(B 
+nil $B$G$"$l$P8=:_$N%X%C%@$rCV49$9$k(B. $BJ#?t$N%X%C%@$,$"$k>l9g$O0l$D$K$J$k(B.
+APPENDP $B$,(B non-nil $B$N$H$-$O%U%#!<%k%I$,DI2C$5$l$k(B."
+  (if appendp
+      (let ((case-fold-search t)
+	    (field-regexp (concat "^" (regexp-quote field-str) ":"))
+	    (beg (point)))
+	(goto (cmail-head-max))
+	(if (re-search-backward field-regexp beg t)
+	    (progn
+	      (forward-line 1)
+	      (while (looking-at "^[ \t]+") (forward-line 1)))))
+    (cmail-delete-field field-str))
+  (insert (concat field-str ": " value-str "\n")))
+
+(defun cmail-trans-date-form (date &optional long-date-format)
   "'Tue, 30 Jun 92 ...'$B$^$?$O(B, '3 Mar 92 ...'$B$H$$$C$?(BDATE$B$NF|IU$+$i(B,
 '92/06/30'$B$H$$$&7A$KJQ49$9$k(B."
   (if (null date)
@@ -228,7 +359,8 @@
       (setq day  "00")))
     (if (< (length day) 2)
 	(setq day (concat "0" day)))
-    (if (> (length year) 2)
+    (if (and (> (length year) 2)
+	     (not long-date-format))
 	(setq year (substring year -2)))
     (if (> (length month) 3)
 	(setq month (substring month 0 3)))
@@ -537,24 +669,56 @@
 
 (defun cmail-virtual-folder-p (folder)
   "FOLDER$B$,2>A[%U%)%k%@$G$"$l$P(Bt$B$rJV$9(B."
-  (and folder (eq (aref folder 0) ?/)))
+  (if folder
+      (or (eq (aref folder 0) ?/)
+	  (if (and
+	       (boundp 'cmail-virtual-folder-check-function)
+	       (functionp cmail-virtual-folder-check-function))
+	      (funcall
+	       cmail-virtual-folder-check-function folder)))))
 
 (defun cmail-clean-virtual-folders (&optional force)
   "$B2>A[%U%)%k%@$r:o=|$9$k(B. $B%f!<%6$K3NG'$r<h$C$?>l9g(Bt$B$rJV$9(B."
-  (let (quit)
+  (let (confirm)
     (if (and (null force) cmail-check-virtual-folder-on-clean)
 	(mapcar (function (lambda (folder)
-			    (cmail-get-folder folder)
-			    (if (> (cmail-mail-counter) 0)
-				(setq quit t))))
+			    (if (not (cmail-volatile-folder-p folder))
+				(setq confirm t))))
 		*cmail-virtual-folders))
-    (and quit
+    (and confirm
 	 (not (y-or-n-p (cmail-get-resource 'clean-virtual-folders-1)))
 	 (cmail-error-resource 'clean-virtual-folders-2))
     (message "")
     (mapcar 'cmail-dismiss-folder *cmail-virtual-folders)
     (setq *cmail-virtual-folders nil)
-    quit))
+    confirm))
+
+(defun cmail-volatile-folder-p (&optional folder)
+  "Return t if FOLDER is a virtual folder, and is marked volatile. A
+virtual folder is by definition a folder loaded in a on-memory
+buffer."
+  (if (cmail-virtual-folder-p folder)
+      (save-excursion
+	(cmail-get-folder folder)
+	(or (and (assq '*cmail-volatile-folder-p (buffer-local-variables))
+		 *cmail-volatile-folder-p)
+	    (= (cmail-mail-counter) 0)))))
+
+(defun cmail-set-folder-to-volatile (&optional folder)
+  "Set FOLDER to be volatile, if it's a virtual folder. If FOLDER is
+not passed in current folder is used."
+  (if (cmail-virtual-folder-p folder)
+      (save-excursion
+	(cmail-get-folder folder)
+	(setq *cmail-volatile-folder-p t))))
+
+(defun cmail-set-folder-to-nonvolatile (&optional folder)
+  "Set FOLDER to be nonvolatile, if it's a virtual folder. If FOLDER
+is not passed in current folder is used."
+  (if (cmail-virtual-folder-p folder)
+      (save-excursion
+	(cmail-get-folder folder)
+	(setq *cmail-volatile-folder-p t))))
 
 (defvar *cmail-prev-folder nil)
 (defvar *cmail-folder-list nil)
@@ -660,18 +824,39 @@ PROMPT$B$KB3$$$FF~NOBT$A$r9T$J$$(B, $BIU2C0z?t$N(BINITIALFOLDER$B$,(Bnon-n
     (let ((fn (cmail-folder-buffer folder))
 	  (file (expand-file-name folder cmail-path))
 	  new)
+      (if (and (boundp 'cmail-mailbox-open-function)
+	       (functionp cmail-mailbox-open-function))
+	  ;; imap4 - if error happens it stops with error.
+	  (progn
+	    (funcall cmail-mailbox-open-function folder
+		     *cmail-confirm-new-folder)
+	    ;; if folder is new, unconditionally create it.  Note:
+	    ;; if a folder is created on the server, it will be
+	    ;; prompted in the imap4 routine. This case is only for
+	    ;; local synchronization.
+	    (if (not (file-exists-p file)) (setq new t)))
+	;; otherwise check the user if we want to create.
+	(and *cmail-confirm-new-folder
+	     (not (file-exists-p file))
+	     (if (yes-or-no-p (cmail-format-resource1 'open-folder-1 folder))
+		 (setq new t)
+	       (cmail-error-resource1 'open-folder-2 folder))))
       (if (cmail-virtual-folder-p folder)
 	  (progn
 	    (set-buffer (get-buffer-create fn))
 	    (setq *cmail-virtual-folders
 		  (cons folder *cmail-virtual-folders)))
-	(and *cmail-confirm-new-folder
-	     (not (file-exists-p file))
-	     (if (yes-or-no-p (cmail-format-resource1 'open-folder-1 folder))
-		 (setq new t)
-	       (cmail-error-resource1 'open-folder-2 folder)))
-	(cmail-make-directory (file-name-directory file))
-	(set-buffer 
+        (cmail-make-directory (file-name-directory file))
+        ;; if this is a new folder, commit the creation anyway, don't
+	;; wait for final save, make it zero length folder. This is to
+	;; make the folders mode happy to show the created folders.
+	(if new
+	    (with-temp-buffer
+	      (erase-buffer)
+	      (insert *cmail-borderline)
+	      (as-binary-output-file
+	       (write-region (point-min) (point-max) file))))
+	(set-buffer
 	 (let ((jam-code-guess-input-coding-system *cmail-file-coding-system)
 	       mc-flag
 	       kanji-fileio-code ; NEmacs $BMQ(B
@@ -715,6 +900,8 @@ PROMPT$B$KB3$$$FF~NOBT$A$r9T$J$$(B, $BIU2C0z?t$N(BINITIALFOLDER$B$,(Bnon-n
       (make-local-variable '*cmail-folders-modified)
       (setq *cmail-folders-modified new)
       (setq buffer-auto-save-file-name nil)
+      (set (make-local-variable '*cmail-folder-name) folder)
+      (set (make-local-variable '*cmail-volatile-folder-p) nil)
       (widen)
       (cmail-make-header-buffer folder)
       (set-buffer-modified-p nil)
@@ -880,7 +1067,6 @@ The value is actually the tail of LIST whose car is ELT."
   "PAGE$BHV9f$N%a%$%k$r:o=|$9$k(B.
 $BIU2C0z?t(BNOECHO$B$,(Bnon-nil$B$N>l9g(B, $B:o=|;~$NJ9$-JV$7$O9T$J$o$J$$!%(B"
   (let ((cnt 0) (beg 0) (end 0)
-	(mailw (get-buffer-window *cmail-mail-buffer))
 	(yon t) buf)
     (cmail-get-folder)
     (goto-char (point-min))
@@ -901,6 +1087,12 @@ The value is actually the tail of LIST whose car is ELT."
 	   (setq yon (y-or-n-p (cmail-get-resource 'delete-mail-1)))))
     (if (not yon)
 	nil
+      ;; imap deletion
+      (if (and (boundp 'cmail-message-delete-function)
+	       (functionp cmail-message-delete-function))
+	  (save-restriction
+	    (narrow-to-region beg end)
+	    (funcall cmail-message-delete-function)))
       (cmail-th-remove page)
       (setq *cmail-deleted t)
       (delete-region beg end)
@@ -967,6 +1159,22 @@ The value is actually the tail of LIST whose car is ELT."
 	(set-buffer hbuf))))
   (run-hooks 'cmail-sync-header-hook))
 
+(defun cmail-update-mark (page mark &optional folder)
+  (save-excursion
+    (let ((regexp (concat "[^" mark "]")))
+      (cmail-get-header folder)
+      (goto-char (point-min))
+      (if (and (re-search-forward (format "^%d " page) nil t)
+	       (looking-at regexp))
+	  (replace-match mark)))))
+
+(defun cmail-retrieve-mark (page &optional folder)
+  (save-excursion
+    (cmail-get-header folder)
+    (goto-char (point-min))
+    (if (re-search-forward (format "^%d " page) nil t)
+	(buffer-substring (point) (1+ (point))))))
+
 (defun cmail-sync-headers ()
   (save-excursion
     (mapcar 'cmail-sync-header
@@ -1006,5 +1214,16 @@ The value is actually the tail of LIST whose car is ELT."
   (fset 'cmail-insert-buffer-substring
 	(symbol-function 'insert-buffer-substring)))
 
+(defun cmail-safe-read (string)
+  (condition-case nil (read (or string "")) (error nil)))
+
+(defun cmail-bufstr-p (bufstr)
+  (and (vectorp bufstr)
+       (buffer-live-p (get-buffer (aref bufstr 0)))
+       (>= (save-excursion
+	     (set-buffer (aref bufstr 0))
+	     (buffer-size))
+	   (max (aref bufstr 1) (aref bufstr 2)))))
+
 (provide 'cmail-misc)
 ;; =========================================================================
