@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-modes.el - cmail summary and readmail mode definition
 ;;;
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Mon Jul 12 10:59:10 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto
@@ -43,6 +43,7 @@
     (setq truncate-lines t)
     (buffer-flush-undo (current-buffer))
     (setq buffer-read-only t)
+    (if (functionp 'hscroll-mode) (hscroll-mode)) ;; ToDo [cmail 7862] by [cmail 9789]
     (run-hooks 'cmail-summary-mode-hook)))
 ;;
 (defun cmail-describe-mode ()
@@ -102,7 +103,7 @@
   (define-key cmail-summary-mode-map "R" 'cmail-reply-include-text)
   (define-key cmail-summary-mode-map "f" 'cmail-followup)
   (define-key cmail-summary-mode-map "F" 'cmail-followup-include-text)
-  (define-key cmail-summary-mode-map "B" 'cmail-resend-bounced-mail)
+  (define-key cmail-summary-mode-map "B" 'cmail-bookmark)
   (define-key cmail-summary-mode-map "@" 'cmail-send-digest)
   (define-key cmail-summary-mode-map "e" 'cmail-edit-index)
   (define-key cmail-summary-mode-map "E" 'cmail-edit)
@@ -166,6 +167,7 @@
   (define-key cmail-summary-mode-map "\C-t" 'cmail-toggle-thread)
   (define-key cmail-summary-mode-map "\C-cmt" 'cmail-toggle-mime-forwarding)
   (define-key cmail-summary-mode-map "\C-cmm" 'cmail-draft-list)
+  (define-key cmail-summary-mode-map "\C-cmb" 'cmail-resend-bounced-mail)
   (define-key cmail-summary-mode-map "\C-c\C-fr" 'cmail-reply-ignore-reply-to)
   (define-key cmail-summary-mode-map "\C-c\C-fR" 'cmail-reply-include-text-ignore-reply-to)
   (define-key cmail-summary-mode-map "\C-c\C-ff" 'cmail-followup-ignore-reply-to)
