@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-reply.el - Mailing, forwarding, and replying functions
 ;;;
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Mon Jul 12 11:01:47 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto
@@ -57,7 +57,8 @@
 				(list self) cmail-self-addresses)))
 	 (dont-cc-me-regexp
 	  (cmail-concat-addr-match-string cmail-dont-cc-me-destinations))
-	 query actions last-message)
+	 (alst cmail-select-from-alist)
+	 query actions last-message to addr trgt pttn full)
 
     (and (if (get-buffer "*mail*") t
 	   (setq noerase nil))
@@ -83,6 +84,7 @@
 	   (setq mail-reply-action replybuffer))
 	  (t
 	   (set-buffer-modified-p nil)
+	   (setq mail-send-actions nil)
 	   (setq last-message (cmail-get-resource 'mail2-5)))))
     (when (not noerase)
       (if (null user)
@@ -96,6 +98,38 @@
 	   (setq cc (cmail-complete-field-to "CC: "))
 	   (setq cc (and (null (string= cc "")) (cmail-address-list cc)))))
 
+    (if (and cmail-select-from-alist
+	     (null cmail-default-from))
+	(save-excursion
+	  (if (not (get-buffer-window *cmail-mail-buffer))
+	      (pop-to-buffer "*mail*")
+	    (select-window (get-buffer-window *cmail-mail-buffer))
+	    (switch-to-buffer "*mail*"))
+	  (let ((case-fold-search t))
+	    (while alst
+	      (setq addr (car (car alst)))
+	      (setq trgt (car (cdr (car alst))))
+	      (setq full (car (cdr (cdr (car alst)))))
+	      (setq alst (cdr alst))
+	      (if (null trgt)
+		  (setq alst nil)
+		(while trgt
+		  (setq pttn (car trgt))
+		  (setq trgt (cdr trgt))
+		  (if (null (string= pttn "DEFAULT"))
+		      (mapcar '(lambda (usr)
+				 (setq to (car usr))
+				 (if (null (string-match pttn to))
+				     nil
+				   (setq alst nil)
+				   (setq trgt nil)))
+			      user)
+		    (setq alst nil)
+		    (setq trgt nil)))))
+	    (make-local-variable 'user-mail-address)
+	    (setq user-mail-address addr)
+	    (setq self (cmail-derive-self-address)))))
+
     ;; CC-me $B=hM}(B
     (if (and user (not (stringp user)) cc-me-p
 	     (null (cmail-address-match self-addr-regexp user))
@@ -120,10 +154,13 @@
     (setq actions mail-send-actions)
 
     (flet ((mail-mode () (cmail-mail-mode)))
-      (if (eq reply 'forward)
-	  ;; 'forward is set in cmail-forward for query ?i
-	  (setq reply nil))
-      (mail noerase user subject reply cc replybuffer nil))
+      (let (*cmail-mime-mail-setup-decode-header-p)
+	(if reply
+	    (setq *cmail-mime-mail-setup-decode-header-p t))
+	(if (eq reply 'forward)
+	    ;; 'forward is set in cmail-forward for query ?i
+	    (setq reply nil))
+	(mail noerase user subject reply cc replybuffer nil)))
 
     (setq mail-send-actions (append actions action))
     (if cmail-default-from
@@ -131,7 +168,29 @@
 	  (goto-char (point-min))
 	  (if (cmail-get-field-values-in-mail-buffer "From")
 	      nil
-	    (insert "From: " cmail-default-from "\n"))))
+	    (insert "From: " cmail-default-from "\n")))
+      (if cmail-select-from-alist           
+	  (save-excursion
+	    (if (null full)
+		nil
+	      (make-local-variable 'user-full-name)
+	      (setq user-full-name full))
+	    (make-local-variable 'user-mail-address)
+	    (setq user-mail-address addr)
+	    (goto-char (point-min))
+	    (cond
+	     ((cmail-get-field-values-in-mail-buffer "From")
+	      nil)
+	     ((and (boundp 'mail-from-style) user-full-name)
+	      (cond
+	       ((eq mail-from-style 'angles)
+		(insert (format "From: %s <%s>\n" (user-full-name) user-mail-address)))
+	       ((eq mail-from-style 'parens)
+		(insert (format "From: %s (%s)\n" user-mail-address (user-full-name))))
+	       (t
+		(insert (format "From: %s\n" user-mail-address)))))
+	     (t
+	      (insert (format "From: %s\n" user-mail-address)))))))
     (setq mail-reply-buffer replybuffer)
     (message last-message)
     (cmail-mail-overload-functions)
@@ -224,13 +283,13 @@
 $B$rL5;k$7$F(B (FROM: $B%U%#!<%k%I$rM%@h$7$F(B) $B%a!<%k%"%I%l%9$r:n@.$9$k!%(B"
   (let ((case-fold-search t)
 	page sbj xnsbj dat from usr nam ref mid rpl ccc buf action
-	lognm rplto frm cc-me-p)
+	lognm rplto frm cc-me-p	status-mark)
 
     ;; You are expected to be on the buffer which contains a message
     ;; you are replying to.
     (if nocmail
 	(run-hooks 'cmail-get-headers-hook)
-      ;; From now on,
+      ;; From here on,
       ;; current buffer is a work buffer which cotains only headers of
       ;; a mail you are replying to.
       (setq page (cmail-get-page-number-from-summary))
@@ -248,8 +307,11 @@
     (setq mid (cmail-get-field-values-in-mail-buffer "Message-Id"))
     (setq from (cmail-get-field-values-in-mail-buffer "From"))
     (if (and (not nocmail) cmail-mail-callback)
-	(setq action (list (cons 'cmail-reply-callback
-				 (list cmail-current-folder page mid)))))
+	(setq action
+	      (list
+	       (cons 'cmail-reply-callback
+		     (list cmail-current-folder
+			   page mid '("Replied"))))))
     (if from
 	(setq lognm (cmail-trans-name-form from 'address)
 	      from (cmail-trans-name-form from 'name))
@@ -283,7 +345,8 @@
 	    (setq usr (cmail-address-list (or rplto frm)))
 	    (setq usr (cmail-dont-reply-to usr self-list))
 	    (setq usr (append usr (cmail-address-list
-				   (cmail-get-field-values-in-mail-buffer "To" ", "))))
+				   (cmail-get-field-values-in-mail-buffer
+				    "To" ", "))))
 	    (setq usr (cmail-unify-dup-addresses usr))
 	    (setq ccc (cmail-address-list
 		       (cmail-get-field-values-in-mail-buffer "CC" ", ")))
@@ -308,7 +371,8 @@
 	  ;; Here, you are on a folder buffer(raw buffer).
 	  (cmail-get-folder)
 	  (cmail-n-page page)
-	  (cmail-set-mail-status "Replied")))
+	  (setq status-mark
+		(cmail-set-mail-status '("Replied")))))
 
     ;;override global var
     (if (not nocmail)
@@ -317,7 +381,7 @@
     (if (and (not nocmail) (not cmail-mail-callback))
 	(save-excursion
 	  (cmail-select-buffer *cmail-summary-buffer)
-	  (cmail-put-mark page "R" "R")))
+	  (apply 'cmail-put-mark page status-mark)))
     (if (or (null sbj) (string= sbj ""))
 	(setq sbj (cmail-get-resource 'reply-2))
       (setq cmail-unify-multi-re-in-subject t))
@@ -355,7 +419,8 @@
     (if yankorigin
 	(progn
 	  (cmail-yank-original nil)
-	  (if (and (featurep 'cmail-use-multi-highlight) cmail-use-multi-highlight)
+	  (if (and (featurep 'cmail-use-multi-highlight)
+		   cmail-use-multi-highlight)
 	      (cmail-modify-keywords-highlight-citation t))))
     (save-excursion
       (if xnsbj
@@ -557,7 +622,8 @@ address string extracted."
     (setq sbj	(buffer-substring (match-beginning 7) (match-end 7)))
     (cmail-get-folder)
     (cmail-n-page page)
-    (setq date (cmail-get-field-values "Date")))
+    (setq msgid (cmail-get-field-values "Message-Id"))
+    (setq date  (cmail-get-field-values "Date")))
 
   (setq *cmail-reply-info
 	(if cmail-reply-heading-format
@@ -583,7 +649,8 @@ address string extracted."
    ((= ctl ?s)
     (substring sbj (string-match "\\(Re:\\s *\\)*" sbj)))
    ((= ctl ?d) date)
-   ((= ctl ?D) (cmail-trans-date-form date))
+   ((= ctl ?D) (cmail-trans-date-form date cmail-reply-long-date-format))
+   ((= ctl ?m) msgid)
    ((= ctl ?n) name)
    ((= ctl ?l) (substring lognm 0 (string-match "[@%]" lognm)))
    ((= ctl ?a) lognm)))
@@ -609,41 +676,57 @@ address string extracted."
   "$B%+%l%s%H5-;v$r%U%)%o!<%I$9$k(B."
   (interactive)
   (let ((page (cmail-get-page-number-from-summary))
-	from subject)
+	from subject action mid)
     (beginning-of-line)
     (if (looking-at "^[ +]*[0-9]+\\^")
 	(progn
 	  (cmail-fixcp)
 	  (cmail-send-digest user))
-	(with-temp-buffer
-	  (let ((*cmail-mail-buffer (buffer-name)))
-      (cmail-fixcp)
-      (cmail-get-folder)
-      (cmail-n-page page)
-      (setq from (cmail-get-field-values "From"))
-      (setq subject (cmail-get-field-values "Subject"))
-      (setq subject (format (cmail-get-resource 'forward-1)
-			    (if (or (null subject) (string= subject ""))
-				(cmail-get-resource 'forward-2)
-			      subject)))
-      (let ((cmail-mime-automatic-preview cmail-mime-forwarding))
-	(save-excursion
-	  (cmail-show-contents page cmail-mime-forwarding)))
-      (cmail-mail user subject 'forward)
-      (set-buffer "*mail*")
-      (save-excursion
-	(when (or (re-search-forward mail-header-separator nil t)
-		  (when (save-excursion
-			  (beginning-of-line 0)
-			  (looking-at mail-header-separator))
-		    (end-of-line 0)
-		    t))
-	  (while (and (search-forward "\n\n" nil 1)
-		      (invisible-p (point)))
-	    (next-visible-point (point))))
-	(if (and cmail-use-mime cmail-mime-forwarding)
-	    (cmail-mime-insert-forward-message)
-	  (cmail-insert-forward-message))))))))
+      (with-temp-buffer
+	(let ((*cmail-mail-buffer (buffer-name)))
+	  (cmail-fixcp)
+	  (cmail-get-folder)
+	  (cmail-n-page page)
+	  (setq from (cmail-get-field-values "From"))
+	  (setq subject (cmail-get-field-values "Subject"))
+	  (setq subject (format (cmail-get-resource 'forward-1)
+				(if (or (null subject) (string= subject ""))
+				    (cmail-get-resource 'forward-2)
+				  subject)))
+	  (setq mid
+		(cmail-get-field-values-in-mail-buffer "Message-Id"))
+	  (let ((cmail-mime-automatic-preview cmail-mime-forwarding)
+		(*cmail-show-content-reason 'forward))
+	    (save-excursion
+	      (cmail-show-contents page cmail-mime-forwarding)))
+	  (if cmail-mail-callback
+	      (setq action
+		    (list
+		     (cons 'cmail-reply-callback
+			   (list cmail-current-folder page
+				 mid '("Forwarded")))))
+	    (save-excursion
+	      (let (status-mark)
+		(cmail-get-folder)
+		(cmail-n-page page)
+		(setq status-mark (cmail-set-mail-status '("Forwarded")))
+		(cmail-select-buffer *cmail-summary-buffer)
+		(apply 'cmail-put-mark page status-mark))))
+	  (cmail-mail user subject 'forward nil nil action)
+	  (set-buffer "*mail*")
+	  (save-excursion
+	    (let ((point (save-excursion
+			   (beginning-of-line)
+			   (re-search-forward mail-header-separator nil t))))
+	      (if (not point)
+		  (beginning-of-line)
+		(goto-char point)
+		(while (and (search-forward "\n" nil 1)
+			    (invisible-p (point)))
+		  (next-visible-point (point)))))
+	    (if (and cmail-use-mime cmail-mime-forwarding)
+		(cmail-mime-insert-forward-message)
+	      (cmail-insert-forward-message))))))))
 
 (defun cmail-alt-forward (arg)
   "$B%+%l%s%H5-;v$r(Bcmail-mime-forwarding$B$N;XDj$G$O$J$$J}K!$G%U%)%o!<%I$9$k(B."
@@ -818,14 +901,16 @@ Prefix arg means don't delete this window."
 (defun cmail-complete-field-to (&optional field)
   "$B%a%$%k$G(B, $B08L>$rF~NO$9$k(B.
 cmail-mailrc$B$+$i(B, $B%0%k!<%W$r=&$C$F$-$FJd40$r9T$J$&(B.
-cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
+cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B. $B$5$i$K(B, cmail-bbdb $B$,%U%#!<%A%c(B
+$B$5$l$F$$$k>uBV$J$i(B, BBDB$B$+$i$b%"%I%l%9$r=&$C$F$-$FJd40%j%9%H$KDI2C$9$k(B. "
   (or field
       (setq field "To: "))
   (let ((cl (cmail-address-complete-list)))
+    (if (featurep 'cmail-bbdb)
+	(setq cl (append cl (bbdb/cmail-get-address-list))))
     (if cl
 	(completing-read field cl)
-      (read-string field))
-    ))
+      (read-string field)) ))
 
 (defun cmail-complete-address ()
   "mail$B%b!<%I$K$*$$$F(Baddress$B$rJd40$9$k(B."
@@ -982,7 +1067,7 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 ;;;
 ;;;
 ;;;
-(defun cmail-reply-callback (folder page message-id)
+(defun cmail-reply-callback (folder page message-id status)
   ;; A callback function should preserve the current buffer
   ;; status, so the save-excursion function is needed here.
   (save-excursion
@@ -991,14 +1076,14 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 	  ;; insert status field in the message in the folder buffer
 	  (cmail-n-page page)
 	  (if (string= (cmail-get-field-values "Message-Id") message-id)
-	      (progn
-		(cmail-set-mail-status "Replied")
+	      (let ((status-mark
+		     (cmail-set-mail-status status)))
 		;; mark the corresponding line in the header buffer
 		(cmail-get-header folder)
 		(goto-char (point-min))
 		(re-search-forward (format "^%d " page) nil t)
 		(delete-char 1)
-		(insert-string "R")
+		(insert-string (car status-mark))
 		;; if the folder is currently viewed, updates the summary
 		(if (eq (cmail-get-folder cmail-current-folder)
 			(current-buffer))
@@ -1007,7 +1092,29 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
 		      (save-excursion
 			(goto-char (point-min))
 			(if (re-search-forward (format "^[ +]*%d" page) nil t)
-			    (cmail-put-mark page "R" "R")))))))))))
+			    (apply 'cmail-put-mark page status-mark)))))))))))
+
+(defun cmail-insert-message (msgseq)
+  "$B8=:_$N%P%C%U%!$N%]%$%s%H0LCV$K8=:_A*Br$5$l$F$$$k$N%U%)%k%@$N(B MSGSEQ 
+$B$G;XDj$5$l$?%a%C%;!<%8$rA^F~$9$k!#<B9T8e$N%]%$%s%H$OA^F~$7$?%F%-%9%H$N(B
+$B:G8e$K0\F0$9$k!#(B"
+  (let ((*cmail-show-content-reason 'forward))
+    (save-window-excursion
+      (cmail-get-folder)
+      (cmail-show-contents msgseq t)))
+  (save-restriction
+    (narrow-to-region (point) (point))
+    (cmail-insert-last-shown-message-raw)
+    (goto-char (point-min))
+    (if (looking-at "^$") (forward-line 1))
+    (while (not (looking-at "^$"))
+      (if (looking-at "^X-cmail-.*:[ \t]*")
+	  (let ((beg (match-beginning 0)))
+	    (forward-line 1)
+	    (while (looking-at "^[ \t]+") (forward-line 1))
+	    (delete-region beg (point)))
+	(forward-line 1)))
+    (goto-char (point-max))))
 
 ;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
@@ -1241,7 +1348,8 @@ cmail-addresses$B$+$i$b(B, $B%"%I%l%9$r=&$C$F$/$k(B."
   (interactive)
   (let ((message (cmail-draft-list-get-draft-message)))
     (cond ((and message (listp message))
-	   (apply (nth 3 message) (list 'open message)))
+	   (apply (nth 3 message) (list 'open message))
+	   (goto-char (mail-text-start)))
 	  ((eq message 'new)
 	   (cmail-mail "" ""))
 	  (t (message "No valid draft message.")))))
