@@ -8,8 +8,7 @@
 	(start-1 . "$B%U%)%k%@(B")
 	(get-spooled-mail-1 . "$B%/%i%C%7%e%U%!%$%k$+$i%a!<%k$rI|85$7$F$$$^$9(B...")
 	(get-spooled-mail-2 . "$B%/%i%C%7%e%U%!%$%k$+$i%a!<%k$rI|85$7$F$$$^$9(B... $B40N;(B")
-	(get-spooled-mail-3 . "POP $B%Q%9%o!<%I(B?: ")
-	(get-spooled-mail-4 . "%s $BMQ$N(B POP $B%Q%9%o!<%I(B?: ")
+	(get-spooled-mail-4 . "%0;s $BMQ$N(B %1;s $B%Q%9%o!<%I(B?: ")
 	(get-spooled-mail-5 . "%s $B$+$i%a!<%k$r<u$1<h$C$F$$$^$9(B...")
 	(get-spooled-mail-6 . "%s $B$+$i%a!<%k$r<u$1<h$C$F$$$^$9(B... $B40N;(B")
 	(get-spooled-mail-7 . "%s $B$+$i%a!<%k$r<h$j9~$s$G$$$^$9(B...")
