@@ -8,8 +8,7 @@
 	(start-1 . "Folder")
 	(get-spooled-mail-1 . "Recovering messages from crash box...")
 	(get-spooled-mail-2 . "Recovering messages from crash box... done")
-	(get-spooled-mail-3 . "POP Password?: ")
-	(get-spooled-mail-4 . "POP Password for %s?: ")
+	(get-spooled-mail-4 . "%1;s Password for %0;s?: ")
 	(get-spooled-mail-5 . "Getting mail from post office %s...")
 	(get-spooled-mail-6 . "Getting mail from post office %s... done")
 	(get-spooled-mail-7 . "Getting new mail from %s...")
