@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-summary.el - making and displaying summaries
 ;;;
-;;;  $Author: iwa $
+;;;  $Author: tmp $
 ;;;  created at: Wed Jul 28 17:09:58 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto.
@@ -32,14 +32,14 @@
 	(dt "00/00/00")
 	(whm "somebody <sombody@somewhere>")
 	(lines 0)
-	(st ?A)
+	(st "A")
 	tmp)
     (goto-char beg)
     (search-forward "\n\n" end 1)
     (setq lines (count-lines (point) end))
     (setq end (point))
     (let ((curbuf (current-buffer))
-	  (idxbuf (get-buffer-create " *cmail-header*")))
+	  (idxbuf (get-buffer-create " *cmail-header*")) status-mark)
       (set-buffer idxbuf)
       (erase-buffer)
       (cmail-insert-buffer-substring curbuf beg end)
@@ -58,14 +58,15 @@
       (setq tmp (cmail-trans-name-form (cmail-get-field-values "From")))
       (if tmp (setq whm tmp))
       (goto-char (point-min))
-      (setq tmp (cmail-trans-date-form (cmail-get-field-values "Date")))
+      (setq tmp (cmail-trans-date-form (cmail-get-field-values "Date")
+				       cmail-summary-long-date-format))
       (if tmp (setq dt tmp))
       (goto-char (point-min))
-      (if (re-search-forward "^X-cmail-status: " end t)
-	  (setq st (char-after (point))))
+      (if (re-search-forward "^X-cmail-status: \\(.+\\)$" end t)
+	  (setq st (elt (cmail-get-mark-symbol (match-string 1)) 1)))
       (kill-buffer idxbuf)
       (set-buffer curbuf))
-    (format "%c %s %s %d [%s]" st dt whm lines sbj)))
+    (format "%s %s %s %d [%s]" st dt whm lines sbj)))
 
 ;; ++++ $B%5%^%j@8@.(B(interactive) +++++++++++++++++++++++++++++++++++++++++++
 (defvar *cmail-folder-curpos nil)
@@ -412,10 +413,16 @@
 	  (set-buffer *cmail-summary-buffer)
 	  (cmail-push-curpos)
 	  (mapcar 'cmail-delete-mail+summary pages)
+	  (if (and (boundp 'cmail-message-expunge-function)
+		   (functionp cmail-message-expunge-function))
+	  (funcall cmail-message-expunge-function folder))
 	  (cmail-diet-folder folder)
 	  (cmail-make-summary cmail-current-folder t)
 	  (cmail-pop-curpos))
       (mapcar 'cmail-delete-mail pages)
+      (if (and (boundp 'cmail-message-expunge-function)
+	       (functionp cmail-message-expunge-function))
+	  (funcall cmail-message-expunge-function folder))
       (cmail-diet-folder folder)))
   (cmail-sync-header folder)
   (cmail-message-resource1 'execute-index-4 folder))
@@ -810,7 +817,14 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (cmail-fixcp)
   (cmail-exec
    '(lambda (page)
-      (cmail-put-mark page "D" "D")) arg)
+      (let (status-mark)
+	(save-excursion
+	  (cmail-get-folder)
+	  (cmail-n-page page)
+	  (setq status-mark
+		(cmail-set-mail-status '("Deleted") '("Hold"))))
+	(apply 'cmail-put-mark
+	       (cmail-get-page-number-from-summary) status-mark))) arg)
   (run-hooks 'cmail-marked-hook)
   (cmail-fixcp)
   (and cmail-mark-delete-show-next
@@ -823,11 +837,12 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (interactive (list cmail-current-folder current-prefix-arg))
   (cmail-message-resource1 'expire-mark-1 folder)
   (let ((cmail-current-folder folder)
-	(n 1) max today date last)
+	(n 1) max today date expdays last)
     (setq today (current-time-string))
     (cmail-get-folder)
     (and delete (setq *cmail-deleted t))
     (setq max (cmail-mail-counter))
+    (setq expdays (cmail-expiry-days folder))
     (while (> max n)
       (cmail-n-page n)
       (if (re-search-forward "^X-cmail-status: U" (cmail-head-max) t)
@@ -835,8 +850,7 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
 	(cmail-n-page n)
 	(setq date (cmail-get-field-values "Date"))
 	(and date 
-	     (> (cmail-days-between today date)
-		(cmail-expiry-days folder))
+	     (> (cmail-days-between today date) expdays)
 	     (if delete
 		 (save-excursion
 		   (cmail-get-header)
@@ -869,7 +883,31 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (cmail-fixcp)
   (cmail-exec
    '(lambda (page)
-      (cmail-put-mark (cmail-get-page-number-from-summary) "H" "H")) arg)
+      (let (status-mark)
+	(save-excursion
+	  (cmail-get-folder)
+	  (cmail-n-page page)
+	  (setq status-mark
+		(cmail-set-mail-status '("Hold") '("Deleted"))))
+	(apply 'cmail-put-mark
+	       (cmail-get-page-number-from-summary) status-mark))) arg)
+  (run-hooks 'cmail-marked-hook)
+  (cmail-fixcp))
+
+(defun cmail-bookmark ()
+  "$B%a%C%;!<%8$N%V%C%/%^!<%/$r%H%0%k$9$k(B. "
+  (interactive)
+  (cmail-fixcp)
+  (let (status-mark (page (cmail-get-page-number-from-summary)))
+    (save-excursion
+      (cmail-get-folder)
+      (cmail-n-page page)
+      (if (cmail-mail-status-p "Bookmarked")
+	  (setq status-mark
+		(cmail-set-mail-status '("Active") '("Bookmarked")))
+	(setq status-mark
+	      (cmail-set-mail-status '("Bookmarked"))))
+      (apply 'cmail-put-mark page status-mark)))
   (run-hooks 'cmail-marked-hook)
   (cmail-fixcp))
 
@@ -879,11 +917,39 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (setq *cmail-current-folder "")
   (cmail-exec
    '(lambda (page)
-      (save-excursion
-	(cmail-get-folder)
-	(cmail-n-page page)
-	(cmail-set-mail-status "Unread"))
-      (cmail-put-mark page "U" "U")) arg)
+      (let (status-mark)
+	(save-excursion
+	  (cmail-get-folder)
+	  (cmail-n-page page)
+	  (cond
+	   ;; if repeat argument is given, add Unread flag.
+	   ((and (integerp arg) (not (eq arg 1)))
+	    (setq status-mark
+		  (cmail-set-mail-status '("Unread"))))
+	   ;; if otherwise, delete standard marks in order.
+	   ((cmail-mail-status-p "Deleted")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Deleted"))))
+	   ((cmail-mail-status-p "Hold")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Hold"))))
+	   ((cmail-mail-status-p "Edited")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Edited"))))
+	   ((cmail-mail-status-p "Bookmarked")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Bookmarked"))))
+	   ((cmail-mail-status-p "Forwarded")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Forwarded"))))
+	   ((cmail-mail-status-p "Replied")
+	    (setq status-mark
+		  (cmail-set-mail-status '("Active") '("Replied"))))
+	   (t
+	    (setq status-mark
+		  (cmail-set-mail-status '("Unread") '("Active"))))))
+	(apply 'cmail-put-mark
+	       (cmail-get-page-number-from-summary) status-mark))) arg)
   (run-hooks 'cmail-marked-hook)
   (cmail-fixcp))
 
@@ -893,11 +959,12 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (setq *cmail-current-folder "")
   (cmail-exec
    '(lambda (page)
-      (save-excursion
-	(cmail-get-folder)
-	(cmail-n-page page)
-	(cmail-set-mail-status "Active"))
-      (cmail-put-mark page " " "A")) arg)
+      (let (status-mark)
+	(save-excursion
+	  (cmail-get-folder)
+	  (cmail-n-page page)
+	  (setq status-mark (cmail-set-mail-status '("Active") '("Unread"))))
+	(apply 'cmail-put-mark page status-mark))) arg)
   (run-hooks 'cmail-marked-hook)
   (cmail-fixcp))
 
@@ -906,15 +973,14 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
   (interactive "p")
   (while (> arg 0)
     (cmail-fixcp)
-    (let ((page (cmail-get-page-number-from-summary)) st1 st2)
+    (let ((page (cmail-get-page-number-from-summary)) status-mark)
       (save-excursion
 	(cmail-get-folder)
 	(cmail-n-page page)
-	(re-search-forward "^X-cmail-status: \\(.\\)" (cmail-head-max) t)
-	(setq st1 (buffer-substring (match-beginning 1) (match-end 1))))
+	(setq status-mark
+	      (cmail-set-mail-status '("Active") '("Deleted" "Hold"))))
       (cmail-fixcp)
-      (setq st2 (if (string= st1 "A") " " st1))
-      (cmail-put-mark page st2 st1))
+      (apply 'cmail-put-mark page status-mark))
     (cmail-go-down 1)
     (setq arg (1- arg)))
   (run-hooks 'cmail-marked-hook)
@@ -933,12 +999,14 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
 		  (progn
 		    (cmail-fixcp)
 		    (= (char-after (1- (point))) ?U)))
-	      (let ((page (cmail-get-page-number-from-summary)))
+	      (let ((page (cmail-get-page-number-from-summary))
+		    status-mark)
 		(save-excursion
 		  (cmail-get-folder)
 		  (cmail-n-page page)
-		  (cmail-set-mail-status "Active"))
-		(cmail-put-mark page " " "A")))
+		  (setq status-mark
+			(cmail-set-mail-status '("Active") '("Unread"))))
+		(apply 'cmail-put-mark page status-mark)))
 	  (forward-line 1))
 	(cmail-message-resource 'catch-up-all-4))
     (message ""))			;erase "(y or n)" line.
@@ -947,22 +1015,26 @@ date-sortable, numeric-sortable$B$N;XDj$K=>$C$F(B, $B%=!<%H2DG=$JF|IU(B,
       (cmail-folders)
     (cmail-fixcp)))
 
-(defun cmail-put-mark (page mark1 mark2)
+(defun cmail-put-mark (page mark1 mark2 &optional folder)
   "PAGE$BHVL\$N(Bmail$B$N%5%^%j$K%^!<%/(BMARK1$B$r(B, $B%$%s%G%C%/%9$K(BMARK2$B$r$D$1$k(B."
   (save-excursion
     (set-buffer *cmail-summary-buffer)
-    (cmail-fixcp)
-    (forward-char -1)
-    (let ((buffer-read-only nil))
-      (looking-at ".")
-      (replace-match mark1 t t))
+    (if (or (null folder) (string= cmail-current-folder folder))
+	(progn
+	  (cmail-fixcp)
+	  (forward-char -1)
+	  (let ((buffer-read-only nil))
+	    (looking-at ".")
+	    (if (not (string= mark1 (match-string 0)))
+		(replace-match mark1 t t)))))
     (if (null mark2)
 	nil
-      (cmail-get-header)
+      (cmail-get-header folder)
       (goto-char (point-min))
       (re-search-forward (format "^%d " page) nil t)
-      (delete-char 1)
-      (insert-string mark2))))
+      (looking-at ".")
+      (if (not (string= mark2 (match-string 0)))
+	  (replace-match mark2 t t)))))
 
 (defun cmail-mark-confirm (mark)
   (let (any (win (current-window-configuration)))
