@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-thread.el - thread related functions
 ;;;
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Fri Jul 14 10:21:00 JST 1995
 ;;;  $Modified: Toshihiko Ueki <toshi@he.kobelcosys.co.jp>
 ;;;  modified at: Sat Nov 23 23:23:23 JST 1997
@@ -26,6 +26,7 @@
 ;; and this notice must be preserved on all copies.
 
 (provide 'cmail-thread)
+(eval-when-compile (require 'cl))
 
 (defun cmail-build-thread ()
   "$B%+%l%s%H%U%)%k%@$N%9%l%C%I%G!<%?$r:n@.$9$k(B. 
@@ -285,15 +286,12 @@ In-Reply-To:$B$,$J$/(BReferences:$B$,$"$l$P(B, $B$=$N:G8e$N%a%C%;!<%8(BID
 	      (or (member msgid family)
 		  (let (family-p)
 		    ;; In-Reply-To $B$G8+$D$+$i$J$$$N$G!"(BReferences $B$rD4$Y$k(B
-		    (mapcar '(lambda (msgid)
-			       (and (member msgid family)
-				    (setq family-p t)	; $B0lB2$@$C$?(B
-				    ;; $B$3$3$G(B mapcar $B$r=*N;$7$?$$$H$3$@$1$I(B...
-				    )
-			       )
-			    (cmail-get-references-list))
-		    (or family-p (setq loop nil))		; $B0lB2$G$O$J$+$C$?(B
-		    ))
+		    (setq family-p
+			  (catch 'found
+			    (dolist (msgid (cmail-get-references-list))
+			      (and (member msgid family)
+				   (throw 'found t)))))	; $B0lB2$@$C$?(B
+		    (or family-p (setq loop nil))))	; $B0lB2$G$O$J$+$C$?(B
 	      (and loop
 		   (setq msgid (cmail-get-field-values "Message-ID"))
 		   (or (member msgid family)
