@@ -1,7 +1,7 @@
 ;;;
 ;;;  cmail-vars.el - cmail variables initialization
 ;;;
-;;;  $Author: MaNakagawa $
+;;;  $Author: tmp $
 ;;;  created at: Mon Jul 12 10:30:57 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1996 Yukihiro Matsumoto.
@@ -281,64 +281,6 @@ message header of the message being processed."))
 
 $B$^$?(B, $B0J2<$K8e<T$NO"A[%j%9%H$NNc$r<($9(B.
 
-((\"N\" (\"X-MAIL-Count\" (\"foo\" \"bar\"))
-      (\"X-Sequence\" (\"hoge\" \"uja\")))
- (\"H\" (\"Header-Field\" (\"glorp\" \"nerp\")))
- (\"F\" (func_name (\"folder_name\" \"etc\"))))
-
-$B$3$3$G(B, \"N\"$B$b$7$/$O(B\"n\"$B$O?tCM$KJQ492DG=$J%U%#!<%k%I$r;}$D%X%C%@$r(B
-$B%=!<%H$K;HMQ$7(B, \"H\"$B$b$7$/$O(B\"h\"$B$O$=$l0J30$N%U%#!<%k%I$r;}$D%X%C%@(B
-$B$r%=!<%H$K;HMQ$9$k(B. $B$^$?(B, \"F\"$B$b$7$/$O(B\"f\"$B$N>l9g$O4X?t$r;XDj$7$F(B, 
-$B$=$NLa$jCM$r%=!<%H$K;HMQ$9$k(B.")
-   (en_US . "\
-*An association list used for the automatic archiving.
-
-Value shall be a list of lisp expressions with the following form.
-
-		(header (pattern . folder))
-
-An element is used as in the order it appears in the list. The
-following is an example association list.
-
-((\"From\" (\"foo\" . \"FOO\")
-         (\"bar\" . \"BAR\"))
- (\"Cc\" (\"glorp\" . \"GLORP\"))
- (\"Subject\" (\"nerp\" . \"NERP\"))
- (\"Header_field\" (\"header_value\" . \"FOLDER_NAME\")))
-
-Instead of a list of pattern/folder list, you may use a function which
-returns a string as the destination folder. At the time the function
-is called, it is coordinated such that the current buffer contains the
-message header of the message being processed."))
-  :type 'sexp
-  :group 'cmail-all-variables
-  :group 'cmail-archive)
-
-(cmail-i18n-defcustom cmail-sort-header-alist nil
-  ((ja_JP . "\
-*$B%U%)%k%@Kh$K%5%^%j$N%=!<%HJ}K!$r;XDj$9$k$?$a$N(B alist.
-
-$B0J2<$N$I$A$i$+$NJB$S$K$J$C$F$$$k(B. $BO"A[%j%9%H$N@hF,$K6a$$$[$I(B
-$BM%@h=g0L$,9b$/$J$C$F$$$k(B.
-
-		(key (folder $B!D(B))
-		(key (pattern (folder $B!D(B)))
-
-$B0J2<$O$=$NA0<T$NO"A[%j%9%H$NNc$G$"$k(B.
-
-((\"S\" (\"foo\" \"bar\" \"hoge\" \"uja\"))
- (\"A\" (\"glorp\" \"nerp\"))
- (\"D\" (\"archive\"))
- (\"R\" (\"folder\" \"folder_name\" \"etc\")))
-
-$B$3$3$G(B, \"S\"$B$b$7$/$O(B\"s\"$B$O%5%V%8%'%/%H=g$N%5%^%j%=!<%H$rI=$7(B,
-\"R\"$B$b$7$/$O(B\"r\"$B$O4XO"$N$"$k%5%V%8%'%/%H=g$N%5%^%j%=!<%H$r(B,
-\"A\"$B$b$7$/$O(B\"a\"$B$OAw$j<g$NL>A0=g$N%5%^%j%=!<%H$r(B, \"D\"$B$b$7$/$O(B
-\"d\"$B$OF|IU=g$N%5%^%j%=!<%H$r(B, \"T\"$B$b$7$/$O(B\"t\"$B$O;~9o$r4^$`F|IU=g$N(B
-$B%5%^%j%=!<%H$r$=$l$>$lI=$9(B.
-
-$B$^$?(B, $B0J2<$K8e<T$NO"A[%j%9%H$NNc$r<($9(B.
-
 ((\"N\" (\"X-MAIL-Count\" (\"foo\" \"bar\"))
       (\"X-Sequence\" (\"hoge\" \"uja\")))
  (\"H\" (\"Header-Field\" (\"glorp\" \"nerp\")))
@@ -688,6 +630,79 @@ Examples:
   :group 'cmail-all-variables
   :group 'cmail-sendmail)
 
+(cmail-i18n-defcustom cmail-select-from-alist nil
+  ((ja_JP . "\
+*From: $B%"%I%l%9$rA*Br$9$k$?$a$N(B alist. 
+$B?7$7$$%a!<%k$N:n@.;~!$JV?.%a!<%k:n@.;~(B, $B%U%)%m!<%"%C%W%a!<%k:n@.;~$K(B
+$B;2>H$5$l$k(B. From: $B%"%I%l%9$H08@h$NO"A[%j%9%H$G(B,
+
+		(from-address (to-pattern ...))
+
+$B$"$k$$$O(B,
+
+		(from-address (to-pattern ...) full-name)
+
+$B$NJB$S$K$J$C$F$$$k(B. $BO"A[%j%9%H$N@hF,$K6a$$$[$IM%@h=g0L$,9b$/$J$C$F$$$k(B.
+$B0J2<$O$=$NO"A[%j%9%H$NNc$G$"$k(B.
+
+((\"foo@bar.org\" (\"@foo.com$\" \"@bar.org$\") \"Foo Bar\")
+ (\"usr@b.ne.jp\" (\".org$\" \".net$\" \".edu$\") \"cmail user\")
+ (\"user@baz.jp\" (\".co.jp$\" \".ac.jp$\" \"to@a.gr.jp$\"))
+ (\"foo@bar.com\" (\"DEFAULT\") \"Default from\"))
+
+$B08@h$N(B \"DEFAULT\" $B$O(B, $B$=$l$h$jM%@h=g0L$N9b$$O"A[%j%9%H$KEv$F$O$^$i$J$$(B
+$B08@h$,$9$Y$F%^%C%A$9$k(B.
+
+\"sendmail.el\" $B$d(B \"feedmail.el\" $B$G;HMQ$5$l$kJQ?t(B \"mail-from-style\"
+$B$NCM$K$h$j(B, From: $B%"%I%l%9$NI=<(7A<0$,7hDj$5$l$k(B.
+
+$BNc(B:
+     \"mail-from-style\" $BL$Dj5A(B    From: Foo Bar <foo@bar.org>
+     'angles                     From: Foo Bar <foo@bar.org>
+     'parens                     From: foo@bar.org (Foo Bar)
+     nil                         From: foo@bar.org
+
+\"cmail-default-from\" $B$O(B, $B$3$NDj5A$h$jM%@hE*$K;HMQ$5$l$k(B.
+")
+   (en_US . "\
+*An association list used for selecting From: address
+when composing new mail, reply mail or follow-up mail.
+
+Value shall be a list of lisp expressions with the following form:
+
+		(from-address (to-pattern ...))
+
+or
+
+		(from-address (to-pattern ...) full-name)
+
+An element is used as in the order it appears in the list. The
+following is an example association list.
+
+((\"foo@bar.org\" (\"@foo.com$\" \"@bar.org$\") \"Foo Bar\")
+ (\"usr@b.ne.jp\" (\".org$\" \".net$\" \".edu$\") \"cmail user\")
+ (\"user@baz.jp\" (\".co.jp$\" \".ac.jp$\" \"to@a.gr.jp$\"))
+ (\"foo@bar.com\" (\"DEFAULT\") \"Default from\"))
+
+The value \"DEFAULT\" is a special To: address pattern that will match
+all addresses.
+
+The style of From: address will be decided by valiable
+\"mail-from-style\" used in \"sendmail.el\" or \"feedmail.el\".
+
+Examples:
+     \"mail-from-style\" undefined    From: Foo Bar <foo@bar.org>
+     'angles                        From: Foo Bar <foo@bar.org>
+     'parens                        From: foo@bar.org (Foo Bar)
+     nil                            From: foo@bar.org
+
+Variable \"cmail-default-from\" takes precedence of this variable
+definition.
+"))
+  :type 'sexp
+  :group 'cmail-all-variables
+  :group 'cmail-sendmail)
+
 (cmail-i18n-defcustom cmail-mail-insert-x-mailer t
   ((ja_JP . "\
 Non-nil $B$N;~!$Aw$k%a%$%k$K(BX-Mailer$B$rIU2C$9$k(B.")
@@ -882,6 +897,8 @@ If nil, cmail prompts for confirmation before deleting the files."))
     (cmail-cite-face                 "SteelBlue"     nil       nil nil)
     (cmail-summary-mode-seen-face    "grey50"        nil       nil nil)
     (cmail-summary-mode-hold-face    "ForestGreen"   nil       nil nil)
+    (cmail-summary-mode-bookmark-face "ForestGreen"  nil       nil nil)
+    (cmail-summary-mode-moreflags-face "DodgerBlue"  nil       nil nil)
     (cmail-summary-mode-reply-face   "DodgerBlue"    nil       nil nil)
     (cmail-summary-mode-unread-face  "blue"          nil       nil nil)
     (cmail-summary-mode-marked-face  "black"         nil       nil nil)
@@ -904,6 +921,8 @@ If nil, cmail prompts for confirmation before deleting the files."))
  (cmail-cite-face                 \"SteelBlue\"     nil          nil  nil)
  (cmail-summary-mode-seen-face    \"grey50\"        nil          nil  nil)
  (cmail-summary-mode-hold-face    \"ForestGreen\"   nil          nil  nil)
+ (cmail-summary-mode-bookmark-face \"ForestGreen\"   nil       nil nil)
+ (cmail-summary-mode-moreflags-face \"DodgerBlue\"   nil       nil nil)
  (cmail-summary-mode-reply-face   \"DodgerBlue\"    nil          nil  nil)
  (cmail-summary-mode-unread-face  \"blue\"          nil          nil  nil)
  (cmail-summary-mode-marked-face  \"black\"         nil          nil  nil)
@@ -926,6 +945,8 @@ If nil, cmail prompts for confirmation before deleting the files."))
  (cmail-cite-face                 \"pink\"           nil          nil  nil)
  (cmail-summary-mode-seen-face    \"grey80\"         nil          nil  nil)
  (cmail-summary-mode-hold-face    \"pink\"           nil          nil  nil)
+ (cmail-summary-mode-bookmark-face \"LightGreen\"   nil       nil nil)
+ (cmail-summary-mode-moreflags-face \"LightSteelBlue\"   nil       nil nil)
  (cmail-summary-mode-reply-face   \"LightSteelBlue\" nil          nil  nil)
  (cmail-summary-mode-unread-face  \"yellow\"         nil          nil  nil)
  (cmail-summary-mode-marked-face  \"skyblue\"        nil          nil  nil)
@@ -1000,6 +1021,32 @@ POP account information is the following format:
      (setq cmail-mail-spools (list cmail-spool-directory))
      (setq cmail-spool-directory nil))
 
+(cmail-i18n-defcustom cmail-mbox-format 'mboxo
+  ((ja_JP . "\
+*$B%a%$%k$N%9%W!<%k%U%!%$%k$N%U%)!<%^%C%H!#(Bmboxo $B$^$?$O(B mboxcl2.
+
+mboxo:   \\nFrom $B$,(B\\n>From $B$K$J$k(B. \\n>From $B$O(B \\n>From $B$N$^$^!#(B
+mboxrd:  \\nFrom $B$,(B\\n>From $B$K$J$k(B. \\n>From $B$b(B \\n>>From $B$K$J$k!#(B
+mboxcl:  mboxo $B$HF1$8$@$,(B Content-Length: $B$,$D$/!#(B
+mboxcl2: \\nFrom $B$O(B\\nFrom $B$N$^$^(B. Content-Length: $B$,$D$/!#(B
+
+cmail $B$G$O(B mboxrd, mboxcl $B$O(B mboxo $B$G07$&!#(B")
+   (en_US . "\
+*Mail spool file format. Either mboxo or mboxcl2.
+
+mboxo:   '\\nFrom ' is quoted to '\\n>From ', '\\n>From ' is not quoted
+         to '\\n>From '.
+mboxrd:  '\\nFrom ' is quoted to '\\n>From '.
+         '\\n>From ' is also quoted to '\\n>>From '.
+mboxcl:  the same as mboxo, but in addition Content-Length: is added.
+mboxcl2: '\\nFrom ' is not quoted, Content-Length: is added.
+
+cmail handles mboxrd, mboxcl as if it is mboxo."))
+  :type '(choice (const :tag "Original BSD format (mboxo)" mboxo)
+		 (const :tag "Content-Length w/o From_ (mboxcl2)" mboxcl2))
+  :group 'cmail-all-variables
+  :group 'cmail-directories)
+
 (defvar cmail-cursor-fast t
   "*$B%5%^%j$G$N%+!<%=%k0\F0$N%9%T!<%I(B. non-nil$B$N>l9g$OAGAa$$%+!<%=%k0\F0(B
 $B$N4X?t$r%P%$%s%I$9$k(B.")
@@ -1076,13 +1123,14 @@ If the value is 0, no backup file will be taken."))
 nil $B$N>l9g$OI=<($r9T$J$o$J$$(B. $B%U%)!<%^%C%H$K4X$9$k5-=R$OJQ?t(B
 cmail-summarize-format $B$r;2>H$N$3$H(B.
 
-S	: $B%5%V%8%'%/%H(B
-s	: $B%5%V%8%'%/%H(B(\"Re: \"$B$,IU$$$F$$$J$$$b$N(B)
-d	: $BF|IU(B
-D	: $BF|IU(B($BNc(B 92/11/01)
-n	: $BL>A0(B
-l	: $B%m%0%$%sL>(B($BNc(B hayasima)
-a	: $B%"%I%l%9(B($BNc(B hayasima@opal.esi.yamanashi.ac.jp)
+S       : $B%5%V%8%'%/%H(B
+s       : $B%5%V%8%'%/%H(B(\"Re: \"$B$,IU$$$F$$$J$$$b$N(B)
+m       : $B%a%C%;!<%8(BID
+d       : $BF|IU(B
+D       : $BF|IU(B($BNc(B 92/11/01)
+n       : $BL>A0(B
+l       : $B%m%0%$%sL>(B($BNc(B hayasima)
+a       : $B%"%I%l%9(B($BNc(B hayasima@opal.esi.yamanashi.ac.jp)
 
 $B$3$NJQ?t$NCM$,J8;zNs$G$J$/4X?t$G$"$C$?>l9g$K$O$=$N4X?t$,8F$S=P$5$l$k(B.
 $B30It%Q%C%1!<%8(B (super-cite $BEy(B)$B$r;HMQ$9$k>l9g$O;HMQ$5$l$J$$(B.")
@@ -1094,6 +1142,7 @@ for more description regarding the format string.
 
 S       : Subject
 s       : Subject (After \"Re: \" is removed)
+m       : Message ID
 d       : Date
 D       : Date (example 92/11/01)
 n       : name
@@ -1107,6 +1156,34 @@ citation such as super-cite is used."))
   :group 'cmail-all-variables
   :group 'cmail-sendmail)
 
+(cmail-i18n-defcustom cmail-reply-long-date-format nil
+  ((ja_JP . "\
+*cmail-reply-heading-format$B$K;XDj$9$k(B%D$B$NG/$N%U%)!<%^%C%H$H$7$F(B
+4$B7eD9$N%U%)!<%^%C%H$rMQ$$$k>l9g$K(Bt$B$r%;%C%H$9$k(B. $B%G%U%)%k%H$O(B2$B7e(B
+$B$NC;$+$$%U%)!<%^%C%H(B.")
+   (en_US . "\
+*When non-nil, use 4-digits year presentation for the format string %D
+in cmail-reply-heading-format, it sets to t.  Default is the short
+paddle format of 2 digits.
+"))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-sendmail)
+
+(cmail-i18n-defcustom cmail-summary-long-date-format nil
+  ((ja_JP . "\
+*cmail-summarize-format$B$K;XDj$9$k(B%d $B$NG/$N%U%)!<%^%C%H$H$7$F(B4$B7e(B
+$BD9$N%U%)!<%^%C%H$rMQ$$$k>l9g$K(Bt$B$r%;%C%H$9$k(B. $B%G%U%)%k%H$O(B2$B7e$N(B
+$BC;$+$$%U%)!<%^%C%H(B.")
+   (en_US . "\
+*When non-nil, use 4-digits year presentation for the format string %d
+in cmail-summarize-format.  Default is the short paddle format of 2
+digits.
+"))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-summary)
+
 (cmail-i18n-defcustom cmail-replyaddress-without-quoted-name t
   ((ja_JP . "\
 *$BJV?.$9$k;~$K(B($BL>A0$J$I$r=|$$$?(B)$B%"%I%l%9$N$_$r%3%T!<$9$k$+(B.
@@ -1530,6 +1607,19 @@ will be disaplayed in the folders mode buffer."))
   :group 'cmail-all-variables
   :group 'cmail-folders)
 
+(cmail-i18n-defcustom cmail-folders-nickname-display nil
+  ((ja_JP . "\
+*Non-nil$B$N;~(B, .cmail-folder-nicknames$B%U%!%$%k$NFbMF$+$i%U%)%k%@$N0&>N$rF@$F$=$l$GCV49$($k(B.
+
+.cmail-folder-nicknames$B$N%U%)!<%^%C%H$O%?%V$^$?$O6uGr$G6h@Z$i$l$?0&>N$H%U%)%k%@L>(B.")
+   (en_US . "\
+*Non-nil means display replace folder names by nicknames from .cmail-folder-nicknames file.
+
+The format of .cmail-folder-nicknames file is nicknames and real names separated by tab/space."))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-folders)
+
 (cmail-i18n-defcustom cmail-use-mime nil
   ((ja_JP . "\
 *MIME $B$r07$&$+$I$&$+(B.
@@ -1859,7 +1949,7 @@ The system must support file permission mode to create the auth file."))
 (defvar *cmail-file-coding-system '*noconv*)
 (defvar *cmail-primary-coding-system '*junet*)
 
-(defvar *cmail-reorder-folder "/ reorder" "$BJB$Y49$(MQ:n6H%U%)%k%@(B")
+(defvar *cmail-reorder-buffer " *cmail-reorder" "$BJB$Y49$(MQ:n6H%P%C%U%!(B")
 
 (defvar *cmail-folders-cache-alist nil
   "$B%U%)%k%@>pJs%-%c%C%7%e(B. $B%(%s%H%j$NFbMF$O0J2<$NDL$j(B:
@@ -1873,6 +1963,31 @@ The system must support file permission mode to create the auth file."))
 $B!&%G%#%l%/%H%j(B
     (DIRNAME DEPTH 'directory FOLDERNUM UNREADNUM . FLAG)")
 
+;; cmail folder $B%P%C%U%!%m!<%+%kJQ?t(B
+(defvar *cmail-folder-name nil)
+(defvar *cmail-volatile-folder-p nil)
+
+;; IMAP4 $BMQ$N(B hooks
+(defvar cmail-content-supply-function nil "hook for imap4")
+(defvar cmail-message-copy-function nil "hook for imap4")
+(defvar cmail-message-delete-function nil "hook for imap4")
+(defvar cmail-message-mark-function nil "hook for imap4")
+(defvar cmail-message-status-function nil "hook for imap4")
+(defvar cmail-message-expunge-function nil "hook for imap4")
+(defvar cmail-mailbox-open-function nil "hook for imap4")
+(defvar cmail-mailbox-delete-function nil "hook for imap4")
+(defvar cmail-mailbox-rename-function nil "hook for imap4")
+(defvar cmail-mailbox-copy-function nil "hook for imap4")
+(defvar cmail-mailbox-search-function nil "hook for imap4")
+(defvar cmail-folders-update-function nil "hook for imap4")
+(defvar cmail-folders-search-function nil "hook for imap4")
+(defvar cmail-summary-message-fetch-function nil "hook for imap4")
+(defvar cmail-virtual-folder-check-function nil "hook for imap4")
+
+;; cmail-show-content $B$r<B9T$9$kM}M3$r%;%C%H$9$kJQ?t(B. forward $B;~(B
+;; $B$K(B imap4 $B$+$iA4FI$_9~$_$9$k$+$rH=CG$5$;$k$?$a$KF3F~(B
+(defvar *cmail-show-content-reason nil)
+
 ;;
 ;; ======= $B:G8e$K%I%-%e%a%s%H$rF~$l49$((B =================================
 ;(cmail-i18n-swap-doc cmail-i18n-doc-language)
