@@ -1,9 +1,9 @@
 ;;;
 ;;;  cmail.el - a mail utility for GNU Emacs
 ;;;
-;;;  $Author: MaNakagawa $
-;;;  $Revision: 1.15 $
-;;;  $Date: 2000/10/03 15:46:23 $
+;;;  $Author: tmp $
+;;;  $Revision: 1.7 $
+;;;  $Date: 2002/10/12 14:25:58 $
 ;;;  created at: Mon Jul 12 10:09:15 JST 1993
 ;;;
 ;;;  Copyright (C) 1992-1998 Yukihiro Matsumoto
@@ -126,6 +126,14 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
     (setq *cmail-current-page 0)
     (if confirm
 	(cmail-visit-folder folder) ;; limit$B$r?R$M$k(B.
+      (if (and (boundp 'cmail-folders-update-function)
+	       (functionp cmail-folders-update-function))
+	  (let ((updated-folders (funcall cmail-folders-update-function)))
+	    (if cmail-archive-on-get
+		(while updated-folders
+		  (cmail-make-summary (car updated-folders))
+		  (cmail-auto-archive nil t)
+		  (setq updated-folders (cdr updated-folders))))))
       (setq *cmail-folder-curpos nil)
       (cmail-make-summary folder)
       (cmail-expand-summary-window)
@@ -141,8 +149,12 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
       (progn
 	(setq *cmail-prev-folder cmail-current-folder)
 	(cmail-folders-get-newmail))
-    (and (cmail-get-newmail)
-	 (cmail-make-summary cmail-inbox-folder))))
+    (if (not (and (boundp 'cmail-summary-message-fetch-function)
+		  (functionp cmail-summary-message-fetch-function)
+		  (funcall cmail-summary-message-fetch-function)))
+	(if (cmail-get-newmail)
+	    (cmail-make-summary cmail-inbox-folder)
+	  (cmail-message-resource 'get-spooled-mail-9)))))
 
 (defun cmail-get-newmail ()
   "$B?7Ce%a%$%k$r<h$C$F$/$k(B."
@@ -201,8 +213,10 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
       (if (cmail-check-crash-box)
 	  (setq mails (+ mails (cmail-gobble-crash-box))))
       (setq account-list (cdr account-list)))
-    (if (= mails 0)
-	(cmail-message-resource 'get-spooled-mail-9))
+;;; imap4 mod: misleading to show this message when new mail
+;;; is retrieved from some imap4 servers.
+;;;    (if (= mails 0)
+;;;	(cmail-message-resource 'get-spooled-mail-9))
     (> mails 0)))
 
 (defun cmail-gobble-crash-box ()
@@ -241,7 +255,7 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
        ((looking-at "From ")
 	(setq d1 "^From "
 	      d2 "\n\nFrom "
-	    fline t))
+	      fline t))
        ((looking-at "\^L\n0, unseen,,\n")
 	(setq d1 "\^L\n0, unseen,,\n"
 	      d2 "\n\^_"))
@@ -255,6 +269,18 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 		    (point)))
 	(setq end (progn
 		    (forward-line 1)
+		    (if (and fline
+			     (eq cmail-mbox-format 'mboxcl2))
+			(let ((case-fold-search t)
+			      header-end)
+			  (re-search-forward "^$" nil t)
+			  (setq header-end (point))
+			  (if (re-search-backward
+			       "^content-length:[ \t]+\\([0-9]+\\)"
+			       beg t)
+			      (goto-char
+			       (+ header-end
+				  (string-to-int (match-string 1)))))))
 		    (if (search-forward d2 nil t)
 			(let ((pos (match-beginning 0)))
 			  (goto-char pos)
@@ -275,7 +301,7 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 	(delete-blank-lines)
 	(insert *cmail-borderline)
       (goto-char (point-min))
-      (cmail-set-mail-status "Unread")
+      (cmail-set-mail-status-if-missing '("Unread"))
       (setq s (cmail-append-mail-to-folder *cmail-arrived-mail-buffer folder))
       (if cmail-incoming-mails-folder
           (let ((*cmail-confirm-new-folder nil))
@@ -325,7 +351,7 @@ prefix$B0z?t$r;XDj$7$?>l9g$O(B, $B?7Ce%a%$%k$NM-L5$r3NG'$;$:$K(B,
 $B30It%W%m%;%9(B movemail $B$rMxMQ$9$k!#(B
 
 PROTO-SPEC $B$O(B (local-file \"filename\") $B$N%j%9%H!#(B"
-  (let ((spool-file (elt proto-spec 1)))
+  (let ((spool-file (elt proto-spec 1)) errors)
     (cmail-message-resource1 'get-spooled-mail-7 spool-file)
     (unwind-protect
       (save-excursion
@@ -360,7 +386,8 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	(password (elt proto-spec 3))
 	(options (elt proto-spec 4))
 	(default-mailhost (getenv "MAILHOST"))
-	(default-movemail (getenv "MOVEMAIL")))
+	(default-movemail (getenv "MOVEMAIL"))
+	errors)
     (if (null password)
 	(setq password (cmail-lookup-password 'pop3 server logname)))
     (if (or (null password) (string= password ""))
@@ -431,6 +458,14 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	  ;; $B%j%9%H$K(B 'skip $B$,F~$C$F$$$?>l9g$O=hM}$7$J$$(B
 	  nil
 	(cond
+	 ;; (protocol "server" "login" "password" "option") $B7A<0$N>l9g(B
+	 ((and (listp spool)
+	       (= (length spool) 5)
+	       (not (eq (car spool) nil))
+	       (not (eq (car spool) t))
+	       (symbolp (car spool)))
+	  (setq res (cons spool res)))
+	 
 	 ;; ("po:foo" "host" "option" "passwd") $B7A<0$N>l9g(B (POP)
 	 ((and (listp spool)
 	       (stringp (car spool))
@@ -484,11 +519,12 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	  (cmail-error-resource 'get-spooled-mail-13)))))
     (reverse res)))
 
-(defun cmail-prompt-for-password (server logname)
+(defun cmail-prompt-for-password (server logname &optional proto-str)
   (let (hmesg pass)
     (setq hmesg
-	  (cmail-format-resource1 'get-spooled-mail-4
-				  (concat logname "@" server)))
+	  (cmail-format-resource 'get-spooled-mail-4
+				 (concat logname "@" server)
+				 (or proto-str "POP")))
     (require 'comint)
     (setq pass (comint-read-noecho hmesg t))
     (message "")
@@ -497,7 +533,9 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 (defun cmail-lookup-password (service server logname)
   (let (password)
     (setq password (cmail-account-db-get server service logname 'password))
-    (if (and (null password) (file-exists-p cmail-auth-file))
+    (if (and (null password)
+	     *cmail-use-auth-file
+	     (file-exists-p cmail-auth-file))
 	(progn
 	  (find-file cmail-auth-file)
 	  (setq buffer-read-only t)
@@ -652,6 +690,8 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 $BIU2C0z?t(BUNDELETE$B$,(Bnon-nil$B$J$i$P(B, $B85(Bmail$B$r:o=|$7$J$$(B."
   (interactive (list (cmail-complete-foldername
 		      (cmail-get-resource 'move-to-folder-1))))
+  (if cmail-folders-nickname-display
+      (setq target (cmail-folders-nickname-realname target)))
   (if (string= cmail-current-folder target)
       (cmail-error-resource 'move-to-folder-2))
   (or (cmail-virtual-folder-p target)
@@ -691,12 +731,27 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 				    (match-end 1)))
     (cmail-get-folder)
     (cmail-n-page page)
-    (setq contents (buffer-substring (point) (cmail-page-max)))
+    (setq contents (list (current-buffer) (point) (cmail-page-max)))
     (set-buffer (get-buffer-create *cmail-arrived-mail-buffer))
     (erase-buffer)
     (if running-emacs-20_3-or-later
 	(cmail-set-buffer-multibyte nil))
-    (insert-string contents)
+    (apply 'insert-buffer-substring contents)
+    ;; imap4 copy - BUG: currently cannot handle fallback situation
+    ;; where a new folder is created on server but not on the local
+    ;; file.
+    (if (and (boundp 'cmail-message-copy-function)
+	     (functionp cmail-message-copy-function))
+	;; server folder will be created if target is imap4 server and
+	;; it does not exist.
+	(progn
+	  (save-excursion
+	    (goto-char (point-min))
+	    (funcall cmail-message-copy-function
+		     cmail-current-folder target
+		     *cmail-confirm-new-folder))
+	  (goto-char (point-max))
+	  (if (not (eolp)) (insert "\n"))))
     (insert-string *cmail-borderline)
     (cmail-append-mail-to-folder *cmail-arrived-mail-buffer target summary))
   (or undelete
@@ -762,6 +817,20 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	(set-buffer *cmail-summary-buffer)))
     (cmail-fixcp)))
 
+(defvar *cmail-last-shown-message-raw nil)
+(defun cmail-insert-last-shown-message-raw ()
+  (if (and (consp *cmail-last-shown-message-raw)
+	   (buffer-live-p (car *cmail-last-shown-message-raw)))
+      (apply 'cmail-insert-buffer-substring
+	     *cmail-last-shown-message-raw)))
+
+(defvar *cmail-last-shown-message-cooked nil)
+(defun cmail-get-last-shown-message-cooked ()
+  (if (and (consp *cmail-last-shown-message-raw)
+	   (buffer-live-p (car *cmail-last-shown-message-raw)))
+      (apply 'cmail-insert-buffer-substring
+	     *cmail-last-shown-message-cooked)))
+
 (defun cmail-show-contents (page &optional all-headers)
   "FOLDER$B$N(BPAGE$BHVL\$N%a%$%k$rI=<($9$k(B."
   (interactive (list (cmail-get-page-number-from-summary)))
@@ -772,7 +841,8 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	  (beg 0)
 	  (end 0)
 	  (pnt 0)
-	  (act nil))
+	  (act nil)
+	  status-mark)
       (cmail-fixcp)
       (forward-char -1)
       (if (= (char-after (point)) ?U)
@@ -781,10 +851,38 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
 	(cmail-select-buffer *cmail-summary-buffer))
       (cmail-get-folder)
       (cmail-n-page page)
-      (if act (cmail-set-mail-status "Active"))
+      (if act
+	  (setq status-mark
+		(cmail-set-mail-status '("Active") '("Unread"))))
       (cmail-n-page page)
       (setq beg (point))
       (setq end (cmail-page-max))
+
+      ;; current buffer is the folder buffer. modifying this buffer
+      ;; will cause change in folder file, and content will be saved
+      ;; as part of folder file.  Try load body if this is a dynamic
+      ;; load folder such as imap4.  After loading, it could swap to a
+      ;; cache buffer if dynamic message loader will not need to save
+      ;; info in the folder file.
+      (if (and (boundp 'cmail-content-supply-function)
+	       (functionp cmail-content-supply-function))
+	  (save-restriction
+	    (narrow-to-region beg end)
+	    (funcall cmail-content-supply-function)
+	    (goto-char (point-min)))
+	(cmail-n-page page))
+
+      ;; current buffer contains raw message, and corresponding body
+      ;; may have been inserted if dynamically loaded.  It may contain
+      ;; partial body in case of imap4 as well.  The raw buffer may be
+      ;; cache buffer instead of the folder buffer. save info for
+      ;; future reference by reply modules.
+      (setq beg (point))
+      (setq end (cmail-page-max))
+      (setq *cmail-last-shown-message-raw
+	    (list (current-buffer) beg end))
+
+      ;; cook it up, bring up *cmail-mail* buffer.
       (save-restriction
         (narrow-to-region beg end)
         (funcall cmail-content-filter-function))
@@ -792,8 +890,18 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
       (cmail-narrow-to-page)
       (setq mode-line-buffer-identification
 	    (format "cmail:   << %s / %d >>" cmail-current-folder page))
+
+      ;; current buffer contains a cooked message for easy viewing,
+      ;; mime view for example. save information for future reference
+      ;; by reply modules.
+      (setq beg (point))
+      (setq end (cmail-page-max))
+      (setq *cmail-last-shown-message-cooked
+	    (list (current-buffer) beg end))
+
+      ;; Go back and adjust summary line cursor.
       (cmail-select-buffer *cmail-summary-buffer)
-      (if act (cmail-put-mark page " " "A"))
+      (if act (apply 'cmail-put-mark page status-mark))
       (if cmail-current-point-mark
 	  (progn
 	    (setq pnt (point))
@@ -812,11 +920,13 @@ PROTO-SPEC $B$O(B (pop3 \"server\" \"logname\" \"password\" \"option\")
       (cmail-fixcp))))
 
 (defun cmail-default-content-filter ()
-  (cmail-select-buffer *cmail-mail-buffer)
-  (setq buffer-read-only nil)
-  (erase-buffer)
-  (cmail-insert-buffer-substring (cmail-folder-buffer cmail-current-folder)
-                           beg end)
+  (let ((contents (current-buffer))
+	(beg (point-min))
+	(end (point-max)))
+    (cmail-select-buffer *cmail-mail-buffer)
+    (setq buffer-read-only nil)
+    (erase-buffer)
+    (cmail-insert-buffer-substring contents beg end))
   (goto-char (point-min))
   (let ((code (detect-coding-region (point-min) (point-max))))
     (if (listp code) (setq code (car code)))
@@ -1051,6 +1161,8 @@ ROT47 will be performed for Japanese text in any case."
   (interactive (list (cmail-complete-foldername (cmail-get-resource 'visit-folder-1))
 		     current-prefix-arg))
   (if (null newfolder) (setq newfolder cmail-current-folder))
+  (if cmail-folders-nickname-display
+      (setq newfolder (cmail-folders-nickname-realname newfolder)))
   (and (string= newfolder cmail-inbox-folder)
        (string= *cmail-prev-folder cmail-inbox-folder)
        (setq *cmail-prev-folder nil))
@@ -1111,6 +1223,9 @@ ROT47 will be performed for Japanese text in any case."
 	(delete-file (expand-file-name folder cmail-path)))
     (if (file-exists-p (expand-file-name (concat folder "~") cmail-path))
 	(delete-file (expand-file-name (concat folder "~") cmail-path))))
+  (if (and (boundp 'cmail-mailbox-delete-function)
+	   (functionp cmail-mailbox-delete-function))
+      (funcall cmail-mailbox-delete-function folder))
   (setq *cmail-folder-list
 	(delete-if '(lambda (lst) (string= (car lst) folder))
 		   *cmail-folder-list))
@@ -1133,20 +1248,29 @@ ROT47 will be performed for Japanese text in any case."
 interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
   (interactive
    (list cmail-current-folder
-	 (cmail-complete-foldername (cmail-format-resource1 'copy-folder-1
-							    cmail-current-folder))))
+	 (cmail-complete-foldername
+	  (cmail-format-resource1 'copy-folder-1
+				  cmail-current-folder))))
   (if (cmail-virtual-folder-p folder2)
       (cmail-error-resource1 'copy-folder-2 folder2))
-  (let ((file1 (expand-file-name folder1 cmail-path))
-	(file2 (expand-file-name folder2 cmail-path)))
-    (if (file-exists-p file2)
-	(cmail-error-resource1 'copy-folder-3 folder2))
-    (cmail-get-folder folder1)
-    (if (not (file-exists-p file1))
-	(cmail-save-folder folder1))
-    (cmail-make-directory (file-name-directory file2))
-    (cmail-save-folder folder1)
-    (copy-file file1 file2))
+  ;; cmail-mailbox-copy-function should return t or stop with error if
+  ;; the copy involves remote folders. Returning nil will result in a
+  ;; local folder copy, which should not occur in the case of remote
+  ;; folders.
+  (if (not (and (boundp 'cmail-mailbox-copy-function)
+		(functionp cmail-mailbox-copy-function)
+		(funcall cmail-mailbox-copy-function
+			 folder1 folder2 *cmail-confirm-new-folder)))
+      (let ((file1 (expand-file-name folder1 cmail-path))
+	    (file2 (expand-file-name folder2 cmail-path)))
+	(if (file-exists-p file2)
+	    (cmail-error-resource1 'copy-folder-3 folder2))
+	(cmail-get-folder folder1)
+	(if (not (file-exists-p file1))
+	    (cmail-save-folder folder1))
+	(cmail-make-directory (file-name-directory file2))
+	(cmail-save-folder folder1)
+	(copy-file file1 file2)))
   (cmail-funcall-if-defined 'cmail-after-copy-folder-function folder1 folder2)
   (message (cmail-format-resource 'copy-folder-4 folder1 folder2)))
 
@@ -1155,10 +1279,17 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
 interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
   (interactive
    (list cmail-current-folder
-	 (cmail-complete-foldername (cmail-format-resource1 'rename-folder-1
-							    cmail-current-folder))))
+	 (cmail-complete-foldername
+	  (cmail-format-resource1 'rename-folder-1
+				  cmail-current-folder))))
   (if (cmail-virtual-folder-p folder2)
       (cmail-error-resource1 'copy-folder-2 folder2))
+  ;; cmail-mailbox-rename-function should stop with error if any of
+  ;; the folders are on a remote server and the rename operation is
+  ;; unsuccessful.
+  (if (and (boundp 'cmail-mailbox-rename-function)
+	   (functionp cmail-mailbox-rename-function))
+      (funcall cmail-mailbox-rename-function folder1 folder2))
   (let ((file1 (expand-file-name folder1 cmail-path))
 	(file2 (expand-file-name folder2 cmail-path)))
     (if (file-exists-p file2)
@@ -1172,7 +1303,8 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
     (rename-file file1 file2))
   (if (string= folder1 cmail-current-folder)
       (cmail-make-summary folder2))
-  (cmail-funcall-if-defined 'cmail-after-rename-folder-function folder1 folder2)
+  (cmail-funcall-if-defined 'cmail-after-rename-folder-function
+			    folder1 folder2)
   (message (cmail-format-resource 'rename-folder-4 folder1 folder2)))
 
 (defun cmail-reorder-folder ()
@@ -1184,31 +1316,50 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
       (cmail-error-resource 'reorder-folder-2))
     (or (cmail-virtual-folder-p folder)
 	(cmail-save-folder cmail-current-folder))
-    (let (page hbuf buf *cmail-prev-folder)
-      (cmail-kill-folder *cmail-reorder-folder t)
+    (let (page summary idx plist buf hbuf fbuf idxept)
+      (setq buf (get-buffer-create *cmail-reorder-buffer))
+      (set-buffer buf)
+      (erase-buffer)
+      (cmail-set-buffer-multibyte nil)
       (cmail-get-folder folder)
-      (setq buf (current-buffer))
-      (cmail-get-header folder)
-      (setq hbuf (current-buffer))
-      (while (re-search-forward "^\\([0-9]+\\) " nil t)
+      (setq fbuf (current-buffer))
+      (goto-char (point-min))
+      (if (re-search-forward *cmail-border)
+	  (setq idxept (point))
+	(error "cmail-reorder-folder: does not appear to be a folder file"))
+      (goto-char (point-min))
+      (setq idx 1)
+      (while (re-search-forward "^\\([0-9]+\\) \\(.*\\)$" idxept t)
 	(setq page (string-to-int (buffer-substring (match-beginning 1)
 						    (match-end 1))))
-	(message (cmail-format-resource 'reorder-folder-3 folder page))
+	(setq summary (buffer-substring (match-beginning 2)
+					(match-end 2)))
+	(setq plist (cons page plist))
 	(save-excursion
-	  (cmail-refile-call page *cmail-reorder-folder t)))
-      (let ((inhibit-quit t))
-	(unwind-protect
-	    (progn
-	      (cmail-get-folder *cmail-reorder-folder)
-	      (copy-to-buffer buf (point-min) (point-max))
-	      (cmail-get-header *cmail-reorder-folder)
-	      (copy-to-buffer hbuf (point-min) (point-max)))
-	  (cmail-kill-folder *cmail-reorder-folder t))))
-    (if (string= folder cmail-current-folder)
-	(cmail-make-summary folder))
-    (cmail-get-folder)
+	  (set-buffer buf)
+	  (insert (format "%d %s\n" idx summary))
+	  (setq idx (1+ idx))))
+      (set-buffer buf)
+      (insert-string *cmail-borderline)
+      (set-buffer fbuf)
+      (setq plist (nreverse plist))
+      (while plist
+	(cmail-n-page (car plist))
+	(message (cmail-format-resource 'reorder-folder-3 folder (car plist)))
+	(append-to-buffer buf (point) (cmail-page-max))
+	(save-excursion
+	  (set-buffer buf)
+	  (insert-string *cmail-borderline))
+	(setq plist (cdr plist)))
+      (set-buffer buf)
+      (copy-to-buffer fbuf (point-min) (point-max))
+      (cmail-make-header-buffer folder)
+      (kill-buffer buf))
+    (cmail-get-folder folder)
     (setq *cmail-pagelist nil)
     (setq *cmail-thread-data nil)
+    (if (string= folder cmail-current-folder)
+	(cmail-make-summary folder))
     (cmail-message-resource1 'reorder-folder-4 folder)))
 
 (defun cmail-saving-folder-match (lt pnt max)
@@ -1330,10 +1481,10 @@ interactive$B$@$H(B, FOLDER1$B$O%+%l%s%H%U%)%k%@$K$J$k(B."
   (interactive)
   (cmail-auto-archive t))
 
-(defun cmail-auto-archive (&optional folder)
+(defun cmail-auto-archive (&optional folder noquery)
   "$B%U%)%k%@$NCf?H$r<+F0E*$K(Brefile$B$9$k(B."
   (interactive "P")
-  (or (string= cmail-current-folder cmail-inbox-folder)
+  (or noquery (string= cmail-current-folder cmail-inbox-folder)
       (y-or-n-p (cmail-format-resource1 'auto-archive-1
 					cmail-current-folder))
       (cmail-error-resource 'auto-archive-2))
