@@ -5,8 +5,8 @@
 ;; Author: Nakagawa Makoto <makoto.nakagawa@jp.compaq.com>
 ;; Keywords: mail
 ;; Created at: Sun May 28 18:00:00 JST 2000
-;; $Date: 2000/12/06 18:29:09 $
-;; $Revision: 1.9 $
+;; $Date: 2002/07/20 23:09:58 $
+;; $Revision: 1.2 $
 
 ;; This file is part of cmail (a mail utility for GNU Emacs)
 
@@ -30,7 +30,7 @@
 ;;; Code:
 (eval-when-compile (require 'static)
 		   (require 'advice)
-		   (require 'cl) ; --> flet
+		   (require 'cl) ; --> flet, dolist
 		   (require 'cmail-vars))
 
 ;;
@@ -136,6 +136,25 @@
   :group 'cmail-all-variables
   :group 'cmail-use-feedmail-group)
 
+(cmail-i18n-defcustom cmail-feedmail-dates-when-queue nil
+  ((ja_JP . "\
+*$BAwIUMQ$X%-%e!<%$%s%0$7$?;~E@$G(B Date $B%U%#!<%k%I$rIU$1$k!#(B")
+   (en_US . "\
+*insert date field when queued to queue, not draft."))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-use-feedmail-group)
+
+(cmail-i18n-defcustom cmail-feedmail-mail-callback-when-queue nil
+  ((ja_JP . "\
+*cmail-mail-callback $B$,(B t $B$G$b!"AwIUMQ$X%-%e!<%$%s%0$7$?;~E@$G(B
+Reply Mark $B$rIU$1$k!#(B")
+   (en_US . "\
+*set reply mark when queued, even if cmail-mail-callback is t."))
+  :type 'boolean
+  :group 'cmail-all-variables
+  :group 'cmail-use-feedmail-group)
+
 (defvar cmail-feedmail-queue-directory
   (concat cmail-feedmail-queue-path
 	  cmail-feedmail-queue-queue-path))
@@ -146,13 +165,14 @@
 ;;
 ;; message resource
 ;;
-(mapcar 'cmail-add-resource
-	`((feedmail-before-queue-prompt
-	   ("Japanese" . "q)$B%-%e!<!"(Bd)$B%I%i%U%H!"(Bi)$BAw?.!'(B")
-	   (t . "q)ueue, d)raft or send i)mmediately: "))
-	  (feedmail-after-send-prompt
-	   ("Japanese" . "$B%I%i%U%H%U%!%$%k(B %s $B$r>C$7$^$9$+!#(B")
-	   (t . "Delete draft file: %s? "))))
+(dolist (resource
+	 `((feedmail-before-queue-prompt
+	    ("Japanese" . "q)$B%-%e!<!"(Bd)$B%I%i%U%H!"(Bi)$BAw?.!'(B")
+	    (t . "q)ueue, d)raft or send i)mmediately: "))
+	   (feedmail-after-send-prompt
+	    ("Japanese" . "$B%I%i%U%H%U%!%$%k(B %s $B$r>C$7$^$9$+!#(B")
+	    (t . "Delete draft file: %s? "))))
+  (cmail-add-resource resource))
 
 ;;
 ;; $BK\BN(B
@@ -204,7 +224,6 @@
 	(if (and is-fqm
 		 (y-or-n-p
 		  (cmail-format-resource1 'feedmail-after-send-prompt
-;		  (format "FQM: Was previously %s; delete that? "
 			  (file-name-nondirectory previous-buffer-file-name))))
 	    (delete-file previous-buffer-file-name)))
     (let ((feedmail-prompt-before-queue-user-alist
@@ -212,25 +231,43 @@
 	     (?I . cmail-feedmail-message-action-send-strong)))
 	  (feedmail-ask-before-queue-prompt
 	   (cmail-format-resource 'feedmail-before-queue-prompt)))
-;	   "q)ueue, d)raft or send i)mmediately: "))
       (flet ((feedmail-tidy-up-slug (slug) (cmail-feedmail-tidy-up-slug slug)))
 	(feedmail-send-it)))))
 
-;; $B$3$l$b(B disable/enable $B$G$-$k$h$&$K$7$H$/!)(B
 (defun cmail-feedmail-message-action-draft ()
   (interactive)
   (flet ((feedmail-tidy-up-slug (slug) (cmail-feedmail-tidy-up-slug slug)))
     (feedmail-message-action-draft)))
+
 (defun cmail-feedmail-message-action-draft-strong ()
   (interactive)
   (let ((feedmail-queue-slug-maker nil)
 	(feedmail-ask-for-queue-slug t))
     (flet ((feedmail-tidy-up-slug (slug) (cmail-feedmail-tidy-up-slug slug)))
       (feedmail-message-action-draft-strong))))
+
 (defun cmail-feedmail-message-action-queue ()
+  "Call \`feedmail-message-action-queue\'.  But insert date field when
+\`cmail-feedmail-dates-when-queue\' is t.  Also execute mail-send-actions
+when \`cmail-feedmail-mail-callback-when-queue\' is t."
   (interactive)
-  (flet ((feedmail-tidy-up-slug (slug) (cmail-feedmail-tidy-up-slug slug)))
-    (feedmail-message-action-queue)))
+  (if cmail-feedmail-dates-when-queue
+      (feedmail-fiddle-date nil))
+  (let ((*mail-send-actions mail-send-actions)
+	(cmail-feedmail-localizer-hook cmail-feedmail-localizer-hook))
+    (flet ((feedmail-tidy-up-slug (slug) (cmail-feedmail-tidy-up-slug slug)))
+      (if (and cmail-mail-callback cmail-feedmail-mail-callback-when-queue)
+	  (remove-hook 'cmail-feedmail-localizer-hook
+		       'cmail-feedmail-set-mail-send-actions-dumper))
+      (feedmail-message-action-queue))
+    (if (and cmail-mail-callback cmail-feedmail-mail-callback-when-queue)
+	(setq mail-send-actions nil) ; this will do no harm even if feedmail-nuke-buffer-after-queue
+	(while *mail-send-actions
+	  (condition-case nil
+	      (apply (car (car *mail-send-actions))
+		     (cdr (car *mail-send-actions)))
+	    (error))
+	  (setq *mail-send-actions (cdr mail-send-actions))))))
 
 (add-hook
  'cmail-derived-mail-mode-setter
@@ -242,7 +279,6 @@
       'cmail-feedmail-message-action-draft-strong)
     (define-key cmail-mail-mode-map "\C-c\C-a"
       'cmail-feedmail-message-action-queue))))
-
 (if running-xemacs
     (add-hook
      'cmail-mail-mode
@@ -278,12 +314,12 @@
 	    ;; $B$"$k>l9g$r9MN8$9$k!#(B
 	    (unless (memq 'cmail-feedmail-localizer cmail-mail-hook)
 	      (add-hook 'cmail-mail-hook 'cmail-feedmail-localizer nil t))
-	    (funcall
-	     cmail-feedmail-queue-runner-mode-setter))))
+	    (funcall cmail-feedmail-queue-runner-mode-setter))))
        auto-mode-alist))
 
 (defvar cmail-feedmail-queue-runner-mode-setter
-  '(lambda (&optional arg)
+  (function
+   (lambda (&optional arg)
 ;	 (when (or (eq 'no-conversion last-coding-system-used)
 ;		   (string-match "\\`raw-text"
 ;				 (symbol-name last-coding-system-used)))
@@ -296,7 +332,8 @@
        (princ (format "find-coding-system is %s \n" coding-system)
 	      (get-buffer-create "*test*"))
        (when (or (eq coding-system 'undecided)
-		 (string-match "raw-text" (symbol-name coding-system)))
+		 (string-match "\\`\\(raw-\\|x-c\\)text\\'"
+			       (symbol-name coding-system)))
 	 (princ (format "detect-coding-system is %s" coding-system)
 		(get-buffer-create "*test*"))
 	 (let ((coding-system
@@ -307,7 +344,7 @@
 	       (setq coding-system default-buffer-file-coding-system))
 	   (decode-coding-region (point-min) (point-max) coding-system)))
        (cmail-mail-mode)
-       (run-hooks 'cmail-mail-hook))))
+       (run-hooks 'cmail-mail-hook)))))
 
 (defun cmail-feedmail-message-action-send ()
   (let ((feedmail-enable-queue nil))
@@ -318,14 +355,15 @@
     (cmail-send)))
 
 (defun cmail-feedmail-localizer ()
-  (mapcar 'make-local-variable
-	  '(send-mail-function 
-	    feedmail-enable-queue
-	    feedmail-queue-directory
-	    feedmail-queue-draft-directory
-	    feedmail-queue-runner-message-sender
-	    feedmail-queue-runner-mode-setter
-	    feedmail-queue-fqm-suffix))
+  (dolist (variable
+	   '(send-mail-function 
+	     feedmail-enable-queue
+	     feedmail-queue-directory
+	     feedmail-queue-draft-directory
+	     feedmail-queue-runner-message-sender
+	     feedmail-queue-runner-mode-setter
+	     feedmail-queue-fqm-suffix))
+    (make-local-variable variable))
   (setq send-mail-function 'feedmail-send-it
 	feedmail-enable-queue cmail-feedmail-enable-queue
 	feedmail-queue-directory cmail-feedmail-queue-directory
@@ -373,6 +411,62 @@
 	    nil t) ; make it local
   (run-hooks 'cmail-feedmail-localizer-hook))
 
+;;
+;; mail-send-actions $B$N1JB32=(B
+;;
+(add-hook 'cmail-feedmail-localizer-hook
+	  'cmail-feedmail-set-mail-send-actions-dumper)
+
+(setq file-name-handler-alist
+      (append file-name-handler-alist
+	      (`(((,(concat "\\" cmail-feedmail-queue-fqm-suffix "$")) .
+		 cmail-feedmail-file-name-handler)))))
+
+(defun cmail-feedmail-set-mail-send-actions-dumper ()
+  (make-local-variable 'write-region-annotate-functions)
+  (setq write-region-annotate-functions
+	(append write-region-annotate-functions
+		'(cmail-feedmail-mail-send-actions-dumper))))
+
+(defun cmail-feedmail-file-name-handler (operation &rest args)
+  (let ((after-insert-file-functions
+	 (append after-insert-file-functions
+		 '(cmail-feedmail-mail-send-actions-loader)))
+	(inhibit-file-name-handlers
+	 (cons 'cmail-feedmail-file-name-handler
+	       (and (eq inhibit-file-name-operation operation)
+		    inhibit-file-name-handlers)))
+	(inhibit-file-name-operation operation))
+    (apply operation args)))
+
+(defconst cmail-feedmail-mail-send-action-header "x-cmail-mail-send-action")
+(defun cmail-feedmail-mail-send-actions-dumper (start end)
+  (mapcar
+   (lambda (action)
+     (cons 1 (concat cmail-feedmail-mail-send-action-header ": "
+		     (prin1-to-string action) "\n")))
+   (reverse mail-send-actions)))
+
+(defun cmail-feedmail-mail-send-actions-loader (bytes)
+  (save-restriction
+    (narrow-to-region (point) (+ (point) bytes))
+    (when (cmail-get-field-values-in-mail-buffer cmail-feedmail-mail-send-action-header)
+      (let (start)
+	(re-search-forward
+	 (concat "^" cmail-feedmail-mail-send-action-header) nil 'move)
+	(goto-char (match-beginning 0))
+	(setq start (point))
+	(make-local-variable 'mail-send-actions)
+	(while (looking-at
+		(concat cmail-feedmail-mail-send-action-header ": "))
+	  (goto-char (match-end 0))
+	  (add-to-list 'mail-send-actions (read (current-buffer)))
+	  (forward-line))
+	(setq bytes (- bytes (- (point) start)))
+	(delete-region (point) start)
+	(goto-char (point-min))
+	bytes))))
+
 ;; feedmail $B$N%G%#%U%)%k%H$O(B feedmail-buffer-to-binmail
 (when (boundp 'send-mail-function)
   (let ((send-mail-function (symbol-name send-mail-function)))
